;
; Program GPS_AX_WX_Tr pro sledovani GPS / WX / SMT-160 / AX.25 ramcu.
; Vznikl prepracovanim kodu UniTracku,WxLcd a AX.25 rx rutin N0QBH z roku 2002.
; Procesor PIC 16F628, krystal 3.579545MHz.
; Program je vychazi z HW zapojeni WX LCD displaye, doplneneho o FX614 modem.
; Tomas, OK1VCF
;
; Prvni smysluplna funkcni verze 1.0 vznikla 22.9.2004 a z GPS funkce umi 
;  dekodovat souradnice,rychlost (km/h),smer,nadmorskou vysku,pocet satelitu,
;  datum, cas. Z vet $GPRMC a $GPGGA - obe musi byt poslany, jinak smycka stale
;  ceka na zbytek dat a nic na LCD nevypisuje.
; LED po zapnuti 3x blikne je-li inicializace OK (1x kratce a 2x dlouze). Pokud
;  zustane svitit, nedari se komunikace s LCD displayem. Po RS-232 je poslana
;  verze firmware a ceka se na NMEA data, prikazy nebo stisky tlacitek.
; Je-li pred poslednim bliknutim STARTOVACI INICIALIZACE stisknuto nektere
;  tlacitko (nebo obe) dojde k prepnuti funkci programu:
;
;  Tlacitko na pinu 17 (LCD): Program prejde do cyklickeho mereni teploty
;			 	Ukoncit tento rezim lze stisknutim tlacitka:
;			 	LCD - program prejde do trasovani GPS+SMT
;				AX25 - program prejde do trasovani audio ramcu
;
;  Tlacitko na pinu  4 (AX25): Program provede reset rychlosti na 4k8 a rezimu
;				mereni teploty na rezim trasovani GPS+SMT,
;				bez ohledu na momentalni stav techto rezimu
;
;  Stiskem obou tlacitek     : Se prepne komunikacni rychlost RS-232
;				z implicitnich 4k8 na 9k6. Tato skutecnost
;				neni nikterak indikovana! 
;
; Implicitni rezim je trasovani GPS+SMT (program umi zjistit pritomnost cidla
;   teploty a kdyz jej nenajde, v dalsich cyklech jiz teplotu nemeri (je-li 
;   GPS FIX OK)). Pokud je GPS FIX nevalidni, program testuje, nebylo-li cidlo
;   teploty pripojeno a pokud ano, zacne teplotu merit.
;  Prepinani obrazovek LCD se provadi stisknutim tlacitka LCD a to pred tiskem
;   dat na LCD.
;  Opustit tento rezim lze stisknutim tlacitka AX25. Program prejde do
;   trasovani AX.25 ramcu (z nf vystupu pripojeneho TRX)
; Behem trasovani AX.25 ramcu lze ovladat beh tlacitky takto:
;  Stisknutim tlacitka AX25 kdykoli po inicializaci (2x blik LED) se vypne
;   kontrola CRC AX.25 ramcu a na LCD/RS232 jsou posilany i ramce s chybnym
;   CRC. Impicitne je CRC kontrola zapnuta.
;  Opustit tento rezim lze stisknutim tlacitka LCD, pak program provede
;   STARTOVACI INICIALIZACI a prejde do rezimu trasovani GPS+SMT
;  LED zde indikuje pritomnost signalu na vstupu FX614. Poblikavajici LED
;   signalizuje "sum", sviti-li LED trvale, je prave dekodovan (a odesilan
;   na LCD + RS-232 AX.25 ramec). Na LCD se posila zdrojova call a zacatek
;   Info Field - celkem max. 32 znaku. Na RS-232 se posila kompletne prvnich
;   190 znaku z AX.25 ramce.
;  V rezimu trasovani AX.25 ramcu program neprovadi cteni z RS-232, nelze tedy
;   zadavat prikazy.
;
; Po prijmu (dekodovani) validnich NMEA dat LED vzdy kratce blikne. 
;  Jsou-li data nevalidni, LED blikne dlouze.
; Validni/invalidni priznak je zobrazovan i na LCD (1.znak 1radky):
;  * validni data
;  ! nevalidni data
;
;	Popis prikazu (po RS-232), znak "^" je <ESC>
;	 ^D - zapis konfigurace do EEPROM
;	 ^U - precteni konfigurace z EEPROM (1. byte je pocet prenasenych bytu,
;	       neni soucasti dat z EEPROM, neposilat ho pri zapisu do EEPROM!)
;	 ^V - precteni verze firmware
;	 ^R - precteni obsahu RAM registru (slouzi pro debug)
;	 ^r - precteni obsahu GPS RAM registru (nactenych dat z GPS prijimace)
;	 ^M - zmeri teplotu
;	 ^W - vypise obsah WX RAM datovych registru
;	 ^w - zapise nasledujici byty do WX RAM datovych registru a posle
;	       je na LCD (zaroven prepne funkci tlacitek na reset minC/maxC
;	       Opustit rezim WX-LCD lze stiskem obou tlacitek, nebo se tak
;	       stane v okamziku, kdy prijde NMEA hlavicka $GPRMC.
;	
;	pin 01 (RA2)  je vstup SMT 160-30
;	pin 02 (RA3)  je RS-232 out
;	pin 03 (RA4)  je RS-232 in
;	pin 04 (MCLR) prepina funkci AX.25 dekoderu / Resetuje funkce
;	pin 05 (VSS)  je zem
;	pin 06 (RB0)  ovlada LED (GPS status, inicializace)
;	pin 07-13 (RB1-RB7) pouzity pro ovladani LCD displaye
;	pin 14 (VDD)  je +5V
;	pin 15-16 (OSC1-2) jsou pro Xtal/rezonator 4MHz
;	pin 17 (RA0)  prepina cyklicky obrazovky LCD (pred zapisem) / SMT Only 
;	pin 18 (RA1)  je vstup signalu z MX614
;	konektor pro PC/GPS je zapojen jako DTE, (pin 2=RXD, 3=TXD, 5=GND)
;
; 28.09.2004 - verze 1.1 
;  Doplnena obsluha z WX-LCD - zobrazeni dat z meteostanicky na LCD.
;  A doplnena indikace aktualni rychlosti RS-232 v obrazovce verze firmware.
;
; 11.12.2004 - verze 1.2
;  Oprava chyby "zdanlive nepripojeneho cidla" pri teplote 0C. Po 4. mereni 
;   byl po odectu offsetu SOUCET logicky nulovy, ale program to pred spustenim
;   dalsiho mereni chybne vyhodnotil jako nepritomnost cidla. Pouze v rezimu
;   GPS, pri validnich NMEA datech.
;
; 08.02.2005 - verze 2.0
;  Typ pripojeneho LCD muze byt bud 2x16 nebo 4x20. Konfigurace se provadi v 
;   EEPROM zapsanim ASCII 2 ci 4 na prislusnou adresu. AX.25 a GPS dekodery
;   pak vyuzivaji vsechny 4 radky...
;
;
	LIST P=PIC16F628 , R=HEX
	include<p16f628.inc>
	

		ORG	0000H

		CALL	LCD_Init	; Inicializace LCD subsystemu
		CLRF	FLAGS_1		; Vynulovat registr priznaku
		GOTO	StartGPS

		; RAM 20h-30h vyuziva WX-LCD nebo AX.25 dekoder
		; RAM 31h-33h vyuziva WX-LCD
		; RAM 34h-35h Hlavni prg
		; RAM 36h-39h vyuziva LCD.ASM
		; RAM 3Ah-3Dh vyuziva SERIAL.ASM
		; RAM 3Eh     Hlavni prg a ostatni - FLAGS_1
		; RAM 3Fh     LCD TYP
		; RAM 40h-40h FREE
		; RAM 41h-6Fh vyuziva NMEA.ASM nebo AX.25 dekoder 
		; RAM 70h-78h vyuziva SMT_160.ASM nebo AX.25 dekoder
		; RAM 79h-7Fh vyuziva WX-LCD nebo AX.25 dekoder
		; RAM         vsechny dalsi banky vyuziva AX.25 dekoder
		; RAM (Banka 1) 20h-21h vyuziva tez obsluha WX
		; RAM (Banka 1) zbytek do 6Fh vyuzito tez pro WX data
		
		ORG	0004H
		goto	int_service	;interrupt vector

AXvers_Data:	
		ADDWF	PCL,F		; Relative addressing
		RETLW	a'A'
		RETLW	a'X'
		RETLW	a'2'
		RETLW	a'5'
		RETLW	a' '
		RETLW	a'P'
		RETLW	a'I'
		RETLW	a'C'
		RETLW	a' '
		RETLW	a'd'
		RETLW	a'e'
		RETLW	a'c'
		RETLW	a'o'
		RETLW	a'd'
		RETLW	a'e'
		RETLW	a'r'
		RETLW	000H

Version_Data:	
		ADDWF	PCL,F		; Relative addressing
		RETLW	a'G'
		RETLW	a'P'
		RETLW	a'S'
		RETLW	a'/'
		RETLW	a'W'
		RETLW	a'X'
		RETLW	a'/'
		RETLW	a'A'
		RETLW	a'X'
		RETLW	a' '
		RETLW	a'T'
		RETLW	a'r'
		RETLW	a' '
		RETLW	a'2'
		RETLW	a'.'
		RETLW	a'0'
		RETLW	000H

No_SMT:	
		ADDWF	PCL,F		; Relative addressing
		RETLW	a'N'
		RETLW	a'o'
		RETLW	a' '
		RETLW	a'S'
		RETLW	a'M'
		RETLW	a'T'
		RETLW	a' '
		RETLW	a'f'
		RETLW	a'o'
		RETLW	a'u'
		RETLW	a'n'
		RETLW	a'd'
		RETLW	a'!'
		RETLW	000H

intro_message
        addwf   PCL,F
        dt      " PIC 16F62x Packet Decoder *"
        dt      " N0QBH (c) 2002 & OK1VCF 2005"
        dt      0x0d,0x0a,0x00



		; NMEA.ASM

AARGB0		EQU	041H		; Registr pro prevod rychlosti
AARGB1		EQU	042H		; Registr pro prevod rychlosti
BARGB0		EQU	043H		; Registr pro prevod rychlosti
NMEA_current_char EQU	044H		; Registr pro NMEA current char
NMEA_mode	EQU	045H		; Registr pro mod vety NMEA
NMEA_index	EQU	046H		; Registr pro NMEA index
NMEA_valid_timer EQU	047H		; Registr pro NMEA valid timer

NMEA_STRING_BASE EQU	048H		; Adresa zacatku NMEA retezce v RAM 
NMEA_STRING_LEN EQU	027H		; Delka NMEA retezce v RAM ->5Fh
ASCII_OFFSET	EQU	01CH		; MIC-E ASCII offset (28d)
LED		EQU	000H		; LED na portu B0

NMEA_Get_100X:				; Znasobi prichozi W 100x
		ANDLW	003H		; jen pro jistotu W<=2
		ADDWF	PCL,F		; Relative addressing
		RETLW	000H		; vraci 0x100
		RETLW	064H		; vraci 1x100
		RETLW	0C8H		; vraci 2x100

NMEA_Get_10X:				; Znasobi prichozi W 10x
		ANDLW	00FH		; jen pro jistotu W<=9
		ADDWF	PCL,F		; Relative addressing
		RETLW	000H		; vraci 0x10 
		RETLW	00AH		; vraci 1x10
		RETLW	014H		; vraci 2x10
		RETLW	01EH		; vraci 3x10
		RETLW	028H		; vraci 4x10
		RETLW	032H		; vraci 5x10
		RETLW	03CH		; vraci 6x10
		RETLW	046H		; vraci 7x10
		RETLW	050H		; vraci 8x10
		RETLW	05AH		; vraci 9x10

Get_NMEA_Title:				; Jakou vetu chceme zpracovavat?
		ANDLW	007H		; jen pro jistotu W<=6
		ADDWF	PCL,F		; Relative addressing
		RETLW	024H		; vraci '$'
		RETLW	047H		; vraci 'G'
		RETLW	050H		; vraci 'P'
		RETLW	052H		; vraci 'R'
		RETLW	04DH		; vraci 'M'
		RETLW	043H		; vraci 'C'
		RETLW	02CH		; vraci ','

Get_NMEA_Title2:			; Jakou vetu chceme zpracovavat?
		ANDLW	007H		; jen pro jistotu W<=6
		ADDWF	PCL,F		; Relative addressing
		RETLW	024H		; vraci '$'
		RETLW	047H		; vraci 'G'
		RETLW	050H		; vraci 'P'
		RETLW	047H		; vraci 'G'
		RETLW	047H		; vraci 'G'
		RETLW	041H		; vraci 'A'
		RETLW	02CH		; vraci ','

NMEA_Get_Mode_Size:			; max pocet znaku pro kazdy mod vety
		ADDWF	PCL,F		; Relative addressing
		RETLW	000H		;  0 $GPRMC
		RETLW	006H		;  1 Time of fix (UTC)
		RETLW	001H		;  2 Fix (A=OK, V=warning)
		RETLW	006H		;  3 Latitude (bez desetine tecky)
		RETLW	001H		;  4 S/N
		RETLW	007H		;  5 Longitude (bez desetine tecky) 
		RETLW	001H		;  6 E/W
		RETLW	003H		;  7 Rychlost v uzlech
		RETLW	003H		;  8 Kurz (vuci true north)
		RETLW	004H		;  9 Datum (den+mesic)
		RETLW	000H		; 10 $GPGGA
		RETLW	006H		; 11 Time of fix
		RETLW	000H		; 12 Latitude
		RETLW	000H		; 13 S/N
		RETLW	000H		; 14 Longitude
		RETLW	000H		; 15 E/W
		RETLW	001H		; 16 Fix quality (0=NOK,1=GPS,2=DGPS)
		RETLW	002H		; 17 Pocet sledovanych satelitu
		RETLW	000H		; 18 Horizontalni odchylka
		RETLW	004H		; 19 Nadmorska vyska-bacha na des.tecku
		RETLW	001H		; 20 v metrech

NMEA_Get_Mode_offset:			; offset pro kazdy mod v NMEA vete
		ADDWF	PCL,F		; Relative addressing
		RETLW	000H		;  0 $GPRMC
		RETLW	000H		;  1 Time of fix (UTC)
		RETLW	006H		;  2 Fix (A-OK, V-warning)
		RETLW	007H		;  3 Latitude (bez desetine tecky)
		RETLW	00DH		;  4 S/N
		RETLW	00EH		;  5 Longitude (bez desetine tecky) 
		RETLW	015H		;  6 E/W
		RETLW	016H		;  7 Rychlost v uzlech
		RETLW	019H		;  8 Kurz
		RETLW	01cH		;  9 Datum (den+mesic)
		RETLW	020H		; 10 $GPGGA
		RETLW	000H		; 11 Time of fix
		RETLW	020H		; 12 Lat
		RETLW	020H		; 13 S/N
		RETLW	020H		; 14 Lon
		RETLW	020H		; 15 E/W
		RETLW	020H		; 16 Fix quality
		RETLW	021H		; 17 Pocet sledovanych satelitu
		RETLW	023H		; 18 Horizontalni odchylka
		RETLW	023H		; 19 Nadmorska vyska-bacha na des.tecku 
		RETLW	027H		; 20 v metrech


NMEA_Read_2_Digit_Binary:		; cte dvoumistne ASCII cislo
		MOVWF	FSR		; z RAMky
		CLRF	NMEA_mode	; zacina s modem 0
		GOTO	NMEA_Read_Binary_Continue

NMEA_Read_3_Digit_Binary:		; cte trimistne ASCII cislo
		MOVWF	FSR		; z RAMky
		MOVF	INDF,W		; vzit rad stovek
		CALL	NMEA_Get_100X	; znasobit ho 100x
		MOVWF	NMEA_mode	; ulozit sem
		INCF	FSR,F		; posunout na dalsi cislo

NMEA_Read_Binary_Continue:		;
		MOVF	INDF,W		; vzit rad desitek
		CALL	NMEA_Get_10X	; znasobit ho 10x
		ADDWF	NMEA_mode,F	; pricist k vysledku
		INCF	FSR,F		; posunout na dalsi cislo
		MOVF	INDF,W		; vzit rad jednotek
		ANDLW	00FH		; odmaskovat jej do binarni podoby
		ADDWF	NMEA_mode,W	; a pricist ho k vysledku
		RETURN			; Return, vysledek je ve W


NMEA_Init:	
		CLRF	NMEA_index	; vynulovat NMEA_index

NMEA_Init_Loop:	
		CALL	NMEA_Reset	;
		CALL	NMEA_Clear_RAM	;
		GOTO	NMEA_Reset	;

NMEA_Decode:				; 
		MOVWF	NMEA_current_char	; ulozit prave prijaty znak
		MOVLW	00AH		; mame uz mod 10, neboli
		SUBWF	NMEA_mode,W	; cteme vetu GGA?
		BTFSC	STATUS,C	; C je 0 kdyz W > NMEA_MODE
		GOTO	NMEA_Decode2	; Ano cteme vetu GGA
		MOVF	NMEA_mode,F	; prichazi nam data?
		BTFSS	STATUS,Z	; Skip if zero
		GOTO	NMEA_Receive	; ano, jdeme je prijmout
		MOVF	NMEA_index,W	; porovnat prichozi znak
		CALL	Get_NMEA_Title	; s hlavickou $GPRMC
		XORWF	NMEA_current_char,W	; pokud to hlavicka
		BTFSS	STATUS,Z	; Skip if zero
		;GOTO	NMEA_Decode2	; neni zkusime druhou vetu
		GOTO	NMEA_Reset	; neni jdeme pryc
		INCF	NMEA_index,F	; je-li z halvicky, posun na dalsi znak
		MOVLW	02CH		; je prijaty znak carka?
		XORWF	NMEA_current_char,W	;
		BTFSS	STATUS,Z	; Skip if zero
		RETURN			; Return, kdyz to neni carka
NMEA_Clear_Spd:				; mame kompletni hlavicku
					; vymaz musi byt rychly, cykl je pomaly 
		MOVLW	NMEA_STRING_BASE + 16H ; od rychlosti
		MOVWF	FSR		; vymazeme stary obsah RAM
		MOVLW	030H		; tim, ze zapiseme nuly
		MOVWF	INDF		; Rychlost
		INCF	FSR,F		; 
		MOVWF	INDF		; Rychlost
		INCF	FSR,F		; 
		MOVWF	INDF		; Rychlost
		INCF	FSR,F		; 
		MOVWF	INDF		; Smer
		INCF	FSR,F		; 
		MOVWF	INDF		; Smer
		INCF	FSR,F		; 
		MOVWF	INDF		; Smer
		INCF	FSR,F		; 
		MOVLW	NMEA_STRING_BASE + 23H ; od vysky
		MOVWF	FSR		; vymazeme stary obsah RAM
		MOVLW	030H		; tim, ze zapiseme nuly
		MOVWF	INDF		; Vyska
		INCF	FSR,F		; 
		MOVWF	INDF		; Vyska
		INCF	FSR,F		; 
		MOVWF	INDF		; Vyska
		INCF	FSR,F		; 
		MOVWF	INDF		; Vyska
		BCF	FLAGS_1, WX_func	; Shodit priznak WX-LCD funkce
		GOTO	NMEA_Next_Mode	; jdeme na dalsi mod

NMEA_Clear_RAM:				; 
		MOVLW	NMEA_STRING_BASE ; od NMEA_STRING_BASE
		MOVWF	FSR		; vymazeme stary obsah RAM

NMEA_Clear_Loop:
		MOVLW	030H		; tim, ze zapiseme nuly
		MOVWF	INDF		; do nasledujicich bytu
		INCF	FSR,F		; od NMEA_STRING_BASE 
		MOVF	FSR,W		; az do
		XORLW	NMEA_STRING_BASE + NMEA_STRING_LEN + 1	;
		BTFSS	STATUS,Z	; Skip if zero
		GOTO	NMEA_Clear_Loop	; jeste nejsme hotovi
		RETURN			; 
NMEA_Decode2:
		;CALL	LED_flash	; bliknout LED
		MOVLW	00BH		; mame uz mod 11, neboli
		SUBWF	NMEA_mode,W	; prichazi nam data?
		BTFSC	STATUS,C	; C je 0 kdyz W > NMEA_MODE
		GOTO	NMEA_Receive	; ano, jdeme je prijmout
		MOVF	NMEA_index,W	; porovnat prichozi znak
		CALL	Get_NMEA_Title2	; s hlavickou $GPGGA
		XORWF	NMEA_current_char,W	; pokud to hlavicka
		BTFSS	STATUS,Z	; Skip if zero
		RETURN			; neni, tak Return
		INCF	NMEA_index,F	; je to znak hlavicky, posun na dalsi
		MOVLW	02CH		; je prijaty znak carka?
		XORWF	NMEA_current_char,W	;
		BTFSS	STATUS,Z	; Skip if zero
		RETURN			; Return, kdyz to neni carka
		GOTO	NMEA_Next_Mode	; jinak jdeme na dalsi mod

NMEA_Receive:
		MOVF	NMEA_current_char,W	;
		XORLW	02CH		; je prave prijaty znak carka?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	NMEA_Next_Mode	; je to carka, jdeme na dalsi mod
		MOVF	NMEA_mode,W	; pro aktualni mod
		CALL	NMEA_Get_Mode_Size	; zjistit pocet jeho znaku 
		XORWF	NMEA_index,W	; a porovnat to
		BTFSC	STATUS,Z	; Skip if not zero
		RETURN			; Return, kdyz uz mame vsechny 

		MOVF	NMEA_mode,W	; je mod 
		XORLW	007H		; roven 7?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	NMEA_Get_Speed_Alti	; mod=7, jdeme na rychlost
		MOVF	NMEA_mode,W	; je mod 
		XORLW	008H		; roven 8?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	NMEA_Get_Speed_Alti	; mod=8, jdeme na smer
		MOVF	NMEA_mode,W	; je mod 
		XORLW	013H		; roven 19?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	NMEA_Get_Speed_Alti	; jdeme na nadmorskou vysku

NMEA_Get_Possition:			; vytahnout pozici a znaky
		MOVF	NMEA_current_char,W	; vzit prijaty znak
		XORLW	02EH		; je to tecka?
		BTFSC	STATUS,Z	; Skip if not zero
		RETURN			; byla to tecka, jdeme na dalsi znak


NMEA_Save_character:			;	
		MOVF	NMEA_mode,W	; pro aktualni mod
		CALL	NMEA_Get_Mode_offset	; zjistit jeho offset
		ADDWF	NMEA_index,W	; pricist ho k indexu a jeste
		ADDLW	NMEA_STRING_BASE	; pricist offset pro NMEA data
		MOVWF	FSR		; na vzniklou adresu
		MOVF	NMEA_current_char,W	; ulozit prijaty znak
		MOVWF	INDF		; do RAMky
		INCF	NMEA_index,F	; posun na dalsi znak
		RETURN			; Return from subroutine


NMEA_Get_Speed_Alti:			; vytahnout rychlost a smer
		MOVF	NMEA_current_char,W	; vzit prijaty znak
		XORLW	02EH		; je to tecka?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	NMEA_Get_Speed_Complete	; je to tecka, koncime
		MOVF	NMEA_mode,W	; pro aktualni mod 
		CALL	NMEA_Get_Mode_offset	; zjistit offset
		ADDLW	NMEA_STRING_BASE	; pricist k adrese
		MOVWF	FSR		; ulozit zjistenou adresu
		INCF	FSR,F		; 
		MOVF	INDF,W		; nakopirovat rad desitek (stovek)
		DECF	FSR,F		;
		MOVWF	INDF		; do stovek (do tisicu)
		INCF	FSR,F		;
		INCF	FSR,F		;
		MOVF	INDF,W		; nakopirovat rad jednotek (desitek)
		DECF	FSR,F		;
		MOVWF	INDF		; do desitek (do stovek)
		INCF	FSR,F		;
		MOVF	NMEA_mode,W	; je aktualni mod 
		XORLW	013H		; nadmorska vyska?
		BTFSS	STATUS,Z	; Skip kdyz je to nadmorska vyska
		GOTO	NMEA_Save_ones	; je to rychlost, jen 3 znaky
		INCF	FSR,F		; je to nadmorska vyska, 4 znaky
		MOVF	INDF,W		; nakopirovat rad (jednotek)
		DECF	FSR,F		;
		MOVWF	INDF		; (do desitek)
		INCF	FSR,F		;

NMEA_Save_ones:				; ulozi prichozi znak do jednotek
		MOVF	NMEA_current_char,W	; prichozi znak 
		MOVWF	INDF		; dat do radu jednotek
		RETURN			; Return from subroutine


NMEA_Get_Speed_Complete:
		MOVF	NMEA_mode,W	; pro aktualni mod
		CALL	NMEA_Get_Mode_Size	; zjistit pocet znaku
		MOVWF	NMEA_index	; a dat je do indexu, nic vic uz se 
		RETURN			; nebude pricitat a Return

NMEA_Get_Mode_Byte			; vrati byte modu dle offsetu ve W
		CALL	NMEA_Get_Mode_offset	; zjistit jeho offset
		ADDLW	NMEA_STRING_BASE	; pricist offset pro NMEA data
		MOVWF	FSR		; vzniklou adresu do FSR
		MOVF	INDF,W		; vzit priznak GPS valid
		RETURN			; ve W vraci 1. byte modu

NMEA_Next_Mode:				; jde na dalsi NMEA mod (slovo vety)
		CLRF	NMEA_index	; vynulovat index (ukazovatko)
		INCF	NMEA_mode,F	; zvysit typ NMEA modu
		;MOVF	NMEA_mode,W	; zjistit, jestli je mod 
		;XORLW	000H		; roven 0
		;BTFSC	STATUS,Z	; Skip if not zero
		;BCF	PORTB,LED	; zhasnout LED
		MOVF	NMEA_mode,W	; pokud ne, 
		XORLW	015H		; je ted roven 21ti?
		BTFSS	STATUS,Z	; Skip if zero
		RETURN			; Return, pokud neni

NMEA_Mode_end:				; Nacteny vsechny mody, konverze dat,
					; zobrazeni dat a pak reset NMEA RX
		CALL	GPS_valid_test	; jsou prijata data validni???
		IORLW	000H		; vratil test binarni nulu?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	GPS_invalid	; je to nula, NEVALIDNI data
		BSF	PORTB,LED	; VALIDNI data, rozsvitit LED
		GOTO	LED_validation_end	; a pokracovat

GPS_invalid:				; data INVALIDNI,
		BSF	PORTB,LED	; rozsvitit LED
		CALL	Start_Mereni	; Zkusit zmerit teplotu
		CALL	LED_flash	; bliknout LED

LED_validation_end:			;
		;CALL	GPS_RAM_send	; pro debug poslat na RS-232
		CALL	SPEED_Convert	; a prepocitat rychlost na km/h
		;CALL	GPS_RAM_send	; pro debug poslat na RS-232
		MOVF	SOUCET, F	; Je pripojene cidlo?
		BTFSS	STATUS, Z	; Je-li SOUCET nulovy, pak cidlo neni
		GOTO	Cidlo_je_merit	; Soucet je nenulovy cidlo mame
		MOVF	poc_mer, F	; soucet=0, ale merili-li jsme 4x, muze
		BTFSC	STATUS, Z	; to znamenat, ze mame teplotu 0C
					; je-li poc_mer=0, merili jsme 4x
Cidlo_je_merit:				; 
		CALL	Start_Mereni	; Cidlo je, zmerit teplotu
		CALL	Zobr_na_LCD	; a zobrazit na LCD
		BCF	FLAGS_1, WX_func	; Shodit priznak WX-LCD funkce


NMEA_Reset:	CLRF	NMEA_index
		CLRF	NMEA_mode
		BCF	PORTB,LED	; zhasnout LED
		RETURN			; Return from subroutine

GPS_valid_test:				; Zjisti zdali je NMEA validni
		MOVLW	002H		; mod pro RMC valid byte
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
		XORLW	041H		; je to 'A'?
		BTFSS	STATUS,Z	; Skip if zero
		RETLW	000H		; NMEA je INVALID
		MOVLW	010H		; mod pro GGA valid byte
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
		XORLW	031H		; je to '1'?
		BTFSC	STATUS,Z	; Skip if not zero
		RETLW	001H		; NMEA je VALIDNI (1 = GPS fix)
		MOVLW	010H		; mod pro GGA valid byte
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
		XORLW	032H		; je to '2'?
		BTFSC	STATUS,Z	; Skip if not zero
		RETLW	001H		; NMEA je VALIDNI (2 = DGPS fix)
		RETLW	000H		; NMEA je INVALID


SPEED_Convert:				; 1uzel = 1.852km/h
					; jednoduse: 1uzel * 15/8 = 1.875km/h
		MOVLW	NMEA_STRING_BASE + 016H	; vzit rychlost
		CALL	NMEA_Read_3_Digit_Binary	; nacist bin. speed
		MOVWF	AARGB0		; ulozit sem
		MOVWF	BARGB0		; a ulozit sem
		CLRF	AARGB1		; vymazat pred nasobenim
		RLF	AARGB0, F	; *2
		RLF	AARGB1, F	; *2
		RLF	AARGB0, F	; *4
		RLF	AARGB1, F	; *4
		RLF	AARGB0, F	; *8
		RLF	AARGB1, F	; *8
		RLF	AARGB0, F	; *16
		RLF	AARGB1, F	; *16
		MOVF	BARGB0, W	; odecist jednou zaklad, aby
		SUBWF	AARGB0, F	; bylo jen *15
		BTFSS	STATUS, C	; C=0 kdyz zaporny vysledek
		DECF	AARGB1, F	; kdyz zaporny vysledek, snizit
		MOVLW	0F8H		; odmaskovat dolni 3 bity pro rotaci
		ANDWF	AARGB0, F	; deleni osmi
                RRF	AARGB1, F	; deleno 2
                RRF	AARGB0, F	; deleno 2
                RRF	AARGB1, F	; deleno 4
                RRF	AARGB0, F	; deleno 4
                RRF	AARGB1, F	; deleno 8
                RRF	AARGB0, F	; deleno 8
		MOVLW	NMEA_STRING_BASE + 016H	; vzit adresu rychlosti
		MOVWF	FSR		; a ulozit do FSR
		CALL	Byte2ASCII	; a prevest ji do ASCII formatu
		RETURN			; konec

Byte2ASCII:				; Prevede obsah AARGB0 do ascii -> INDF
		MOVLW	02FH		; ASCII hodnota pred 0 (0=30H)
		MOVWF	INDF		; do radu stovek
		MOVF	AARGB0, W	; vzit prvotni hodnotu AARGB0
Byte2ASCII100s:				; 
		INCF	INDF, F		; pricist jednicku
		MOVWF	AARGB0		; dat odecteny byte do AARGB0
		MOVLW	d'100'		; nejdrive rad stovek
		SUBWF	AARGB0, W	; odecist 100 od rychlosti
		BTFSC	STATUS,C	; C je 0 kdyz W > AARGB0
		GOTO	Byte2ASCII100s	; jeste nejaka stovka zbyva
		MOVF	INDF, W		; vzit rad stovek
		XORLW	030H		; je to ascii nula?
		MOVLW	020H		; pripravit mezeru
		BTFSC	STATUS, Z	; preskoc, pokud neni ascii nula
		MOVWF	INDF		; do stovek mezeru, kdyz < 100km/h
		INCF	FSR, F		; jdeme na rad desitek
		MOVLW	02FH		; ASCII hodnota pred 0 (0=30H)
		MOVWF	INDF		; do radu desitek
		MOVF	AARGB0, W	; vzit prvotni hodnotu AARGB0
Byte2ASCII10s:				; 
		INCF	INDF, F		; pricist jednicku
		MOVWF	AARGB0		; dat odecteny byte do AARGB0
		MOVLW	d'10'		; ted rad desitek
		SUBWF	AARGB0, W	; odecist 100 od rychlosti
		BTFSC	STATUS,C	; C je 0 kdyz W > AARGB0
		GOTO	Byte2ASCII10s	; jeste nejaka stovka zbyva
		INCF	FSR, F		; jdeme na rad jednicek
		MOVF	AARGB0, W	; vzit prvotni hodnotu AARGB0
		ADDLW	030H		; pricist k jednickem ASCII offset
		MOVWF	INDF		; ulozit do radu jednicek
		RETURN			; hotovo


		; SMT_160.ASM

SMT_IN		EQU	002H		; Port RA2 s cidlem SMT-160
stridaL		EQU	070H		; Registr pro mereni teploty
stridaH		EQU	071H		; Registr pro mereni teploty
SOUCET		EQU	072H		; Registr pro vysledek mereni
countL		EQU	073H		; Registr pro merici smycku
countH		EQU	074H		; Registr pro merici smycku
teplH		EQU	075H		; Registr s ASCII teplotou
teplL		EQU	076H		; Registr s ASCII teplotou
teplZ		EQU	077H		; Registr s ASCII teplotou
poc_mer		EQU	078H		; Registr poctu mereni

Mereni_Init				;
		MOVLW	004H		; merime 4x
		MOVWF	poc_mer		; zde je pocitadlo
		CLRF	SOUCET		; vynulovat namerenou hodnotu
		CLRF	stridaL		; vynulovat citace
		CLRF	stridaH		; stridy
		RETURN			;


SMT_to_ASCII				; Prevede namerenou hodnotu do ASCII
		MOVLW	a'+'		; vzit znamenko
		MOVWF	teplZ		; a dat ho sem
		MOVLW	d'68'		; odecist 68 C
		SUBWF	SOUCET, F	; od spocitane teploty
		BTFSC	STATUS, C	; je-li zaporna
		GOTO	SMT_kladne	; teplota kladna
		COMF	SOUCET, F	; spocitat doplnek,
		MOVLW	a'-'		; vzit znamenko
		MOVWF	teplZ		; a dat ho sem

SMT_kladne				; SOUCET = ABS hodnota odectene tepl.
		MOVF	SOUCET, W	; absolutni hodnotu odectene
		MOVWF	AARGB0		; teploty dat sem
		MOVLW	countH		; sem ulozit vyslednou teplotu v ASCII
		MOVWF	FSR		; a ulozit do FSR
		CALL	Byte2ASCII	; a prevest ji do ASCII formatu
		RETURN			; konec

Start_Mereni				; Mereni SMT-160 teploty 
		MOVF	poc_mer, F	; Uz jsme merili
		BTFSS	STATUS, Z	; 4x??
		GOTO	Zmer_SMT	; jeste ne, zmerit
		CALL	Mereni_Init	; Priprava pro nove mereni

Zmer_SMT				; 
		CALL	Mereni_SMT	; Zmerit
		MOVF	stridaH, W	; vzit vysledek
		MOVWF	SOUCET		; ulozit ho sem
		DECF	poc_mer, F	; odecist pocet mereni
		RETURN			; a hotovo

Mereni_SMT	; Rutina popsana Ondrejem Vitousem na HW serveru
		MOVLW	020H		; merime 108ms (425*32*8us)
		MOVWF	countL		; 435*32*8=1360=3520 +
		MOVLW	036H		; cyklus pro countL
		MOVWF	countH		; = 3620h

		;CLRF	stridaL		; vynulovat citace
		;CLRF	stridaH		; stridy
		BCF	STATUS, 2	; vynulovat Z
		
smycka9		nop			; 1 kazdy 256 cykl = 9us
smycka8		BTFSC	PORTA, SMT_IN	; 1/2
		INCF	stridaL, F	; 2	zvysit, pokud je SMT_IN log. 1
		BTFSC	STATUS, 2	; 3/4 
		INCF	stridaH, F	; 4	zvysit, pokud pretekl reg
			
		DECFSZ	countL, F	; 5/6
		GOTO	smycka9		; 6+1=7
		DECFSZ	countH, F	; 7/8
		GOTO	smycka8		; 7+1=8

		;BCF	STATUS, 0	; 1 vynulovat CY
		;RLF	stridaL, F	; 1 a odroloval
		;RLF	stridaH, F	; 1 v ramci 
		;RLF	stridaL, F	; 1 dvoubajtoveho
		;RLF	stridaH, F	; 1 registru
		;RLF	stridaL, F	; 1 32*, tzn mame
		;RLF	stridaH, F	; 1 pocet pulstupnu

		RETURN			


		; SERIAL.ASM

serialDelayCount EQU	03AH		; Registr odecet zpozdeni serial delay
serialData	EQU	03BH		; Registr pro byte k RX/TX
serialTemp	EQU	03CH		; Registr pro cosi?
serialCount	EQU	03DH		; Registr pro pocet bitu k odeslani
SERIAL_IN	EQU	004H		; PORT A4 je RS232 input
SERIAL_OUT	EQU	003H		; PORT A3 je RS232 output
;SERIAL_SPEED	EQU	005H		; PORT A5(MCLR) = RS232 rychlost 0=vyssi
;		RS-232 delay time should be: ((ClockSpeed/4/SerialSpeed)-14)/3
;DELAY_TIME_Q	EQU	050H		; rychlost 9600 @10MHZ
;DELAY_TIME_S	EQU	0A9H		; rychlost 4800 @10MHZ
;DELAY_TIME_sQ	EQU	01AH		; zkracena rychlost 9600 @10MHZ
;DELAY_TIME_sS	EQU	038H		; zkracena rychlost 4800 @10MHZ
;DELAY_TIME_Q	EQU	01EH		; rychlost 9600 @4MHZ
;DELAY_TIME_S	EQU	040H		; rychlost 4800 @4MHZ
;DELAY_TIME_sQ	EQU	00AH		; zkracena rychlost 9600 @4MHZ
;DELAY_TIME_sS	EQU	014H		; zkracena rychlost 4800 @4MHZ
DELAY_TIME_Q	EQU	01AH		; rychlost 9600 @3.579545MHz
DELAY_TIME_S	EQU	039H		; rychlost 4800 @3.579545MHz
DELAY_TIME_sQ	EQU	008H		; zkracena rychlost 9600 @3.579545MHz
DELAY_TIME_sS	EQU	013H		; zkracena rychlost 4800 @43.579545MHz

serial_Delay:
		MOVWF	serialDelayCount	; ulozit W do delay pocitadla
		NOP				; Helps with high speed	

serial_Delay_Loop:				; odecitaci smycka
		DECFSZ	serialDelayCount,F	; odecist
		GOTO	serial_Delay_Loop	; jeste jsme neodcetli vse
		RETURN				; odecteno, Return


serial_Init:
		BSF	STATUS,RP0		; Select page 1 (80H - FFH)
		MOVLW	b'10100111'		; Prescaler -> TMR0,
		MOVWF	OPTION_REG		; 1:256, rising edge
		BSF	TRISA,SERIAL_IN		; Nastavit vstup
		BCF	TRISA,SERIAL_OUT	; Nastavit vystup
		BCF	STATUS,RP0		; Select page 0 (00H - 7FH)
		BCF	PORTA,SERIAL_OUT	; Vystup nastavit do LOW
		RETURN				; Return from subroutine


serial_Transmit:				; odesle byte po RS232
		MOVWF	serialData		; ulozit byte pro odeslani
		MOVLW	00AH			; 8 bitu 1 stop a 1 start bit
		MOVWF	serialCount		;
		BCF	STATUS,C		; Clear carry flag

serial_Transmit_Loop:
		BCF	PORTA,SERIAL_OUT	; zaciname s LOW na vystupu
		BTFSS	STATUS,C		; Skip if carry set
		BSF	PORTA,SERIAL_OUT	; kdyz posilame 0, poslat HIGH
		
		MOVLW	DELAY_TIME_S		; pomalejsi rychlost RS232
		BTFSC	FLAGS_1,SERIAL_SPEED	; Nebo ma byt rychlost vyssi?
		MOVLW	DELAY_TIME_Q		; vetsi rychlost RS232
		
		CALL	serial_Delay		; cekat dobu bitu
		BSF	STATUS,C		; Set carry flag
		RRF	serialData,F		; odrotovat bity odesilaneho 
		DECFSZ	serialCount,F		; bytu, odecist odeslany bit
		GOTO	serial_Transmit_Loop	; jeste neodeslany vsechny
		RETURN				; odeslano, Return

serialTXcr:
		MOVLW	00DH			; Odeslat CR
		GOTO	serial_SEND_byte		
		RETURN			

serialTXlf:
		MOVLW	00AH			; Odeslat LF
		GOTO	serial_SEND_byte
		RETURN


serialTXsp:	
		MOVLW	020H			; Odeslat mezeru
		GOTO	serial_SEND_byte
		RETURN

serial_SEND_byte:				; Odeslat znak ve W
		GOTO	serial_Transmit		; skok na odeslani

serial_Receive:					;
		BCF	serialData,00		; pripravime prijaty bit jako 0
		BTFSS	PORTA,SERIAL_IN		; pokud neprijimame data
		RETLW	000H			; tak navrat
		MOVLW	009H			; jinak cteme
		MOVWF	serialCount		; 9 bitu

		MOVLW	DELAY_TIME_sS		; zkracena pomala RS232
		BTFSC	FLAGS_1,SERIAL_SPEED	; Nebo ma byt rychlost vyssi?
		MOVLW	DELAY_TIME_sQ		; zkracena rychla RS232
		CALL	serial_Delay		; cekat delku bitu

serial_Receive_Loop:				; 
		BCF	STATUS,C		; Clear carry flag
		BTFSS	PORTA,SERIAL_IN		; pokud prichazi 0
		BSF	STATUS,C		; Set carry flag
		RRF	serialData,F		; odrotovat serial data bits

		MOVLW	DELAY_TIME_S		; pomalejsi rychlost RS232
		BTFSC	FLAGS_1,SERIAL_SPEED	; Nebo ma byt rychlost vyssi?
		MOVLW	DELAY_TIME_Q		; vetsi rychlost RS232
		
		CALL	serial_Delay		; cekat delku bitu
		NOP				; keep timing w/ transmit
		DECFSZ	serialCount,F		; mame vsechny bity?
		GOTO	serial_Receive_Loop	; jeste ne, tak dalsi
		MOVF	serialData,W		; prijaty byte do W

serial_Receive_Stop:				;
		BTFSC	PORTA,SERIAL_IN		; cekat na 0 na serial in
		GOTO	serial_Receive_Stop	; je tam stale 1 (stop bit)
		BSF	serialData,00		; bit 0 nastavit na 1
		RETURN				; Return from subroutine


serial_Receive_BYTE:				; prijme byte z RS232 portu
		CALL	serial_Receive		; nacist byte
		BTFSS	serialData,00		; nacten? 
		GOTO	serial_Receive_BYTE	; ne, data zadna neprichazeji
		RETURN				; Byte nacten, Return
		
		
		; EEPROM.ASM

EEPROM_Read_Byte:			; cte byte z adresy W EEPROM
		;ANDLW	03FH		; odmaskovat bity 6 a 7 pro 64Bytu RAM
		ANDLW	07FH		; odmaskovat bity 7 pro 128Bytu RAM
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVWF	EEADR		; zapsat adresu, ze ktere budeme cist
		BSF	EECON1,RD	; bude se cist z EEPROM 
		MOVF	EEDATA,W	; a nacteme byte
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		RETURN			; Return from subroutine


EEPROM_Write_Byte:			; zapise byte na adresu W EEPROM
		;ANDLW	03FH		; odmaskovat bity 6 a 7 pro 64Bytu RAM
		ANDLW	07FH		; odmaskovat bit 7 pro 128Bytu RAM
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVWF	EEADR		; zapsat adresu, kam budeme zapisovat
		BCF	INTCON,GIE	; Generalni zakaz preruseni
		BSF	EECON1,WREN	; Povolit EEPROM write
		MOVLW	055H		; REQUIRED WRITE SEQUENCE START
		MOVWF	EECON2		; Write 55h
		MOVLW	0AAH		; 
		MOVWF	EECON2		; Write AAh
		BSF	EECON1,WR	; Set WR bit, zapis spusten
		BCF	EECON1,WREN	; Zakazat EEPROM write
		BSF	INTCON,GIE	; Preruseni opet povolit

EEPROM_Write_Complete_Loop:		; ceka az se zapise do EEPROM
		BTFSC	EECON1,WR	; uz se byte zapsal?
		GOTO	EEPROM_Write_Complete_Loop	; jeste ne, cekat
		BCF	STATUS,RP0	; zapsano, Select page 0 (00H - 7FH)
		RETURN			; Return from subroutine


		; LCD.ASM

;
; tybytov komunikace LCD displae s radiem HD44780 a mikroadie PIC16F84.
;
; Dne 19.5.2001
;
; Pavel Prochzka
; hacesoft@mujmail.cz
; www.prochazka.zde.cz
; Z portu B je voln pouze bit 0. Ostatn vvody portu B pouv LCD display.
;Port A je cel voln.
;K ovldn Portu B bitu 0 pouvejte instrukce BCF PORTB,0 a BSF PORTB,0.
;K nastaven tohoto bitu jako vstupn/vstupn pouijte nsledujc sekvenci:
; 	BCF nebo BSF reg_trisb,0	
;	MOVf	reg_trisb,w
;	TRIS	PORTB
; Za podmnky e pob v peruen.
; Pokud nebudete pouvat peruen sta aby bity co pouv LCD display 
;byli nastavenu jako vstupn. Ale muste konfiguraci zapsat i do registru 
;'reg_trisb' jinak po proveden jakhokoliv pkazu s LCD displaem bude 
;bit 0 porty B nastaven jako vstupn (zkladn nastaven). Pokud vm 
;bude stait aby bit 0 portu B byl jen vstupn, tak nic nemuste
;konfigurovat.
; S portem A mete nakldat jakkoliv.
;
; Zapojeni LCD displae:		vvod PICu	vvod LCD
;				PORTB 1  	RS - vyber reg.
;				PORTB 2		RW - read/write
;				PORTB 3		E  - enable
;				PORTB 4		DB4
;				PORTB 5		DB5
;				PORTB 6		DB6
;				PORTB 7		DB7
; Vvody DB0 a DB3 z LCD displae pipojit na 0V.
 
 
LCD_TEMP	EQU	036H	; hodnota, ktera se ma poslat do LCD
reg_trisb	EQU	037H	; hodnota, ktera se vysle na TRISB
 
DELAY		EQU	038H	; Used in DELAYxxx routines
X_DELAY		EQU	039H	; Used in X_DELAYxxx routines
LCD_TYP		EQU	03fH	; Typ LCD
 
#define		e		PORTB,3
#define		rw		PORTB,2
#define		rs		PORTB,1
 
SCR_SEL		EQU	000H		; SW na RA0 prepina LCD obrazovky

LCD_Init:				; Inicializace LCD a LED
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVLW	00FH		; cely port B
		MOVWF	TRISB		; Nastavit jako vystup
		;BSF	TRISB,LED	; Nastavit vystup
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		BSF	PORTB,LED	; Vystup nastavit do HIGH
		CALL	LCDINIT		; vlastni inicializace LCD
		BCF	PORTB,LED	; Vystup nastavit do LOW
		BCF	FLAGS_1, LCDscr	; LCD obrazovka prvni
		MOVLW	EE_LCD_TYP	; adresa typu LCD
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_TYP		; a ulozit ji sem
		RETURN			; hotovo

		
; inicializace LCD displae.
LCDINIT
	clrf	reg_trisb                   
    	MOVLW   0x01E
    	CALL    X_DELAY500      ; 30 * 0.5mS = 15mS
 
;***************** Nastaveni 8-bitove komunikace ***************************
 
	MOVLW   0x030           ; 8-bit-interface
    	call	SENDPRIKAZ
 
;*************** Nastaveni 4-bitove komunikace *****************************
 
	MOVLW   0x020           ; 4-bit-interface
	call	SENDPRIKAZ
 
;***************************************************************************
	MOVLW   0x028           ; 4-bit-interface, 2-lines
	CALL    LCDPUTCMD
	MOVLW   0x008           ; disp.off, curs.off, no-blink
	CALL    LCDPUTCMD
	MOVLW   0x001		;lcd smazat
	CALL    LCDPUTCMD
	MOVLW   0x00C           ; disp.on, curs.off
	CALL    LCDPUTCMD
	MOVLW   0x006		; Inkrementace, normalni mod.
	CALL    LCDPUTCMD
	RETURN
 
SENDPRIKAZ
	CALL    LCDBUSY			; Wait for LCD to be ready
	BCF     rw
	BCF     rs
	BSF     e
	call	Zapis_w			; Zapise reg. W na PortB.
	BCF     e
	return
 
; Ceka az Display dodela prikaz. Obsah reg. W se touto rutinou nemeni!
LCDBUSY
	movwf   LCD_TEMP		; Uloz reg. W do LCD_TEMP
	movf	reg_trisb,w
	andlw	b'00001111'
	iorlw	b'11110000'		; Byty 4 az 7 brany B jsou vstupni.
	movwf	reg_trisb
	;tris	PORTB
	BSF	STATUS,RP0		; Select page 1 (80H - FFH)
	MOVWF	TRISB			; Nastavit PORT B
	BCF	STATUS,RP0		; Select page 0 (00H - 7FH)
lcd_busy
	BCF     rs
	BSF     rw
	BSF     e
	movf    PORTB,w         	; Read busy flag + DDram address
	BCF     e
	BSF     e
	BCF     e
	andlw   b'10000000'		; Test na BF priznak.
	BTFSS   STATUS,Z
	goto    lcd_busy
	bcf     rw
	movf	reg_trisb,w
	andlw	b'00001111'        	; Byty 4 az 7 brany B jsou vystupni.       		
	movwf	reg_trisb
	;tris	PORTB
	BSF	STATUS,RP0		; Select page 1 (80H - FFH)
	MOVWF	TRISB			; Nastavit PORT B
	BCF	STATUS,RP0		; Select page 0 (00H - 7FH)
	movf    LCD_TEMP,w		; Obnov reg. W
	return
 
; Posle znak na Display.
LCDPUTCHAR
	CALL    LCDBUSY             ; Wait for LCD to be ready
	BCF     rw
	BSF     rs
	BSF     e
	call	Zapis_w			; Zapise reg. W na PortB.
	BCF     e
 
	BSF     e
	swapf	LCD_TEMP,f
	call	Zapis_w			; Zapise reg. W na PortB.
	BCF     e
	RETURN
 
; Posle prikaz na Display.
LCDPUTCMD
	call    SENDPRIKAZ
	BSF     e
	swapf	LCD_TEMP,f
	call	Zapis_w			; Zapise reg. W na PortB.
	BCF     e
    	return
 
; Prekopiruje obsah reg. LED_TEMP bity 4 az 7 na PORTB bity 4 az 7.
Zapis_w
	movlw	b'00001111'
	andwf	PORTB,f		
; Vynuluje bity 4 az 7 portub, vysledek ulozi do portb.
	movlw	b'11110000'
	andwf	LCD_TEMP,w	
; Vynuluje bity 0 az 3 lcd_temp, vysledek ulozi do w.
	iorwf	PORTB,f		
; OR mezi w a potrb, vysledek ulozit do portb.
	return
 
;***********************************  a 500uS delay @ 4MHz X-tal
DELAY500
	MOVLW       D'165'              ; +1        1 cycle
	MOVWF       DELAY               ; +2        1 cycle
DELAY500_LOOP
	DECFSZ      DELAY, F            ; step1     1 cycle
	GOTO        DELAY500_LOOP       ; step2     2 cycles
DELAY500_END
	RETURN                          ; +3        2 cycles
 
;***********************************  a delay of 'W' * 500mS
X_DELAY500
	MOVWF       X_DELAY             ; +1        1 cycle
X_DELAY500_LOOP
	CALL        DELAY500            ; step1     wait 500uSec
	DECFSZ      X_DELAY, F          ; step2     1 cycle
	GOTO        X_DELAY500_LOOP     ; step3     2 cycles
X_DELAY500_END
	RETURN                          ; +2        2 cycles
 
Speed_LCD:				; zobrazi rychlost na LCD
		MOVLW	b'11001101'	; nastavit kursor na 14. byte 2. radku
 		CALL	LCDPUTCMD	;
		BTFSC	FLAGS_1,SERIAL_SPEED	; Je rychlost RS-232 4k8?
		CALL	S9k6toLCD		; ne, je 9k6 
		BTFSC	FLAGS_1,SERIAL_SPEED	; Je rychlost RS-232 4k8?
		GOTO	Speed_LCD_end	; hotovo
		CALL	S4k8toLCD		; ano, je 4k8 
Speed_LCD_end:				;
		RETURN			; hotovo
		
SMT_Yes_No_LCD:				; 
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
 		CALL	LCDPUTCMD
		MOVF	SOUCET, F	; Je soucet
		BTFSC	STATUS, Z	; nenulovy?
		GOTO	Send_No_SMT_LCD	; Pokud je SOUCET=0
		CALL	SMT_to_ASCII	; 
		MOVLW	a'S'		; 
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVLW	a'M'		; 
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVLW	a'T'		; 
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVLW	a':'		; 
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		CALL	Space2LCD	; Poslat mezeru
		MOVF	teplZ, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVF	teplH, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVF	teplL, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVLW	0DFH		; znak stupne
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'C'		; 
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		CALL	Space2LCD	; Poslat mezeru
		CALL	Space2LCD	; Poslat mezeru
		CALL	Space2LCD	; Poslat mezeru
		RETURN			; navrat

Send_No_SMT_LCD:			; odesle po startu text  
		CLRF	temp		; na LCD, vynulovat temp_reg
Send_No_SMT_LCD_loop:			; po RS232 odesle text
		MOVF	temp,W		; offset bytu textu
		CALL	No_SMT		; ziskat byte textu
		IORLW	000H		; je to binarni nula?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	No_SMT_just_send_LCD	; je, cely text poslan
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		INCF	temp,F		; posun na dalsi byte
		GOTO	Send_No_SMT_LCD_loop	; a zpracovat ho
No_SMT_just_send_LCD:			;
		RETURN			; navrat

Send_Version_LCD:			; odesle po startu text verze 
		MOVLW	b'00000001'	; clear a kursor na 1. byte 1. radku
 		CALL	LCDPUTCMD
		CLRF	temp		; na LCD, vynulovat temp_reg
Send_Version_LCD_loop:			; po RS232 odesle text verze FW
		MOVF	temp,W		; offset bytu verze
		CALL	Version_Data	; ziskat byte verze
		IORLW	000H		; je to binarni nula?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Version_just_send_LCD	; je, cely text poslan
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		INCF	temp,F		; posun na dalsi byte
		GOTO	Send_Version_LCD_loop	; a zpracovat ho
Version_just_send_LCD:			;
		RETURN			; navrat

Send_AXvers_LCD:			; odesle po startu text verze 
		MOVLW	b'00000001'	; clear a kursor na 1. byte 1. radku
 		CALL	LCDPUTCMD
		CLRF	temp		; na LCD, vynulovat temp_reg
Send_AXvers_LCD_loop:			; po RS232 odesle text verze FW
		MOVF	temp,W		; offset bytu verze
		CALL	AXvers_Data	; ziskat byte verze
		IORLW	000H		; je to binarni nula?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	AXvers_just_send_LCD	; je, cely text poslan
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		INCF	temp,F		; posun na dalsi byte
		GOTO	Send_AXvers_LCD_loop	; a zpracovat ho
AXvers_just_send_LCD:			;
		RETURN			; navrat


Zobr_na_LCD:				; Zobrazi pripraveny WX retezec
Screen_Select:				; Nejdriv zjistit stav tlacitka
		BTFSC	PORTA, SCR_SEL	; je stisknuto tlacitko?
		GOTO	Scr_selected	; neni, pryc
		BTFSS	FLAGS_1, LCDscr	; Je nastavena obrazovka 2?
		GOTO	SetScr2		; Neni, nastavime 2
		BCF	FLAGS_1, LCDscr	; Neni, nastavime 1
		GOTO	Scr_selected	;
SetScr2:				;
		BSF	FLAGS_1, LCDscr	; Nastavime 2
		GOTO	Scr_selected	;

Scr_selected:				; 
		MOVLW	b'10000000'	; nastavit kursor na 1. byte 1. radku
 		CALL	LCDPUTCMD
		CALL	GPS_valid_test	; jsou prijata data validni???
		IORLW	000H		; vratil test binarni nulu?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	GPS_invLCD	; je to nula, NEVALIDNI data
		MOVLW	a'*'		; VALIDNI data
		BTFSC	STATUS,Z	; Skip if not zero
GPS_invLCD:				;
		MOVLW	a'!'		; NEVALIDNI data
		CALL	LCDPUTCHAR	; Poslat na LCD

		MOVF	LCD_TYP, W	; 2x16 nebo 4x20??
		SUBLW	a'4'		; a'4' = 4x20
		BTFSC	STATUS, Z	; preskoc, pokud 2x16
		GOTO	Zobr_LCD_1	; pokud 4x20, zobrazime obe scr...

		BTFSC	FLAGS_1, LCDscr	; Je nastavena obrazovka 1?
		GOTO	Zobr_LCD_2	; 

Zobr_LCD_1:				; *LA'LA.LAx rrrKm
					; LON'LO.LOy vvvvm
					; 1. radek
		MOVLW	003H		; mod pro Latitude
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
		MOVLW	0DFH		; znak stupne
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Dot2LCD		; Poslat tecku na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
		MOVLW	007H		; mod pro Speed
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Km2LCD		; Poslat Km na LCD
					; 2. radek
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
 		CALL	LCDPUTCMD
		MOVLW	005H		; mod pro Longitude
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
		MOVLW	0DFH		; znak stupne
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Dot2LCD		; Poslat tecku na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
		MOVLW	013H		; mod pro Altitude
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD

		MOVF	LCD_TYP, W	; 2x16 nebo 4x20??
		SUBLW	a'4'		; a'4' = 4x20
		BTFSC	STATUS, Z	; preskoc, pokud 2x16
		GOTO	Zobr_LCD_2	; pokud 4x20, zobrazime obe scr...
		RETURN			; hotovo

Zobr_LCD_2:				; *HH:MI:SS  rrrKm
					; cc+DD.MM.nn sss'
		MOVF	LCD_TYP, W	; 2x16 nebo 4x20??
		SUBLW	a'4'		; a'4' = 4x20
		BTFSS	STATUS, Z	; preskoc, pokud 4x20
		GOTO	Zobr_LCD_2x	; pokud 2x16
		MOVLW	b'10010100'	; nastavit kursor na 1. byte 3. radku
 		CALL	LCDPUTCMD	; 

Zobr_LCD_2x:				; 
					; 1. radek (nebo 3. radek)
		MOVLW	000H		; mod pro Time of fix
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
		MOVLW	a':'		; dvojtecka
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
		MOVLW	a':'		; dvojtecka
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
		MOVLW	007H		; mod pro Speed
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Km2LCD		; Poslat Km na LCD

					; 2. radek (nebo 4. radek)
		MOVF	LCD_TYP, W	; 2x16 nebo 4x20??
		SUBLW	a'4'		; a'4' = 4x20
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
		BTFSC	STATUS, Z	; preskoc, pokud 2x16
		MOVLW	b'11010100'	; nastavit kursor na 1. byte 4. radku
 		CALL	LCDPUTCMD	; 

		MOVF	SOUCET, F	; Je pripojene cidlo?
		BTFSC	STATUS, Z	; Je-li SOUCET nulovy, pak cidlo neni
		GOTO	Zobr_FIX	; Cidlo neni, zobrazit Fix
		MOVF	poc_mer, F	; Uz mame zmerenou celou teplotu?
		BTFSC	STATUS, Z	; Pokud ano, poc_mer=0
		CALL	SMT_to_ASCII	; Prevest teplotu do ASCII
		MOVF	teplH, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVF	teplL, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		MOVF	teplZ, W	;
 		CALL	LCDPUTCHAR	; Poslat znak na LCD
		GOTO	FIX_SMT_OK	;

Zobr_FIX:				; misto teploty zobrazi GPS FIX
		MOVLW	002H		; mod pro RMC valid byte
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	010H		; mod pro GGA valid byte
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
		GOTO	FIX_SMT_OK	;

FIX_SMT_OK:				; Fix nebo teplota zobrazeny, dale
		MOVLW	009H		; mod pro Date of fix
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Dot2LCD		; Poslat tecku na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Dot2LCD		; Poslat tecku na LCD
		MOVLW	011H		; mod pro pocet satelitu
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	Space2LCD	; Poslat mezeru na LCD
		MOVLW	008H		; mod pro kurz
		CALL	NMEA_Get_Mode_Byte	; ziskat 1. Byte modu
 		CALL	LCDPUTCHAR	; Poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
 		CALL	NextByte2LCD	; Dalsi BYTE poslat na LCD
		MOVLW	0DFH		; znak stupne
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo

NextByte2LCD				; posle na LCD byte z FSR po pricteni 1
		INCF	FSR, F		; posun na dalsi BYTE
		MOVF	INDF,W		; vzit jej
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo

Space2LCD				; posle mazeru na LCD
		MOVLW	a' '		; mezera
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo
		
Dot2LCD					; posle tecku na LCD
		MOVLW	a'.'		; tecka
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo
		

S4k8toLCD				; posle 4k8 na LCD
		MOVLW	a'4'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'k'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'8'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo


S9k6toLCD				; posle 9k6 na LCD
		MOVLW	a'9'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'k'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'6'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo

Km2LCD					; posle Km na LCD
		MOVLW	a'K'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
m2LCD					; posle m na LCD
		MOVLW	a'm'		; 
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo

AX_LCD_Reset:				;
		MOVLW	b'00000001'	; clear a kursor na 1. byte 1. radku
 		CALL	LCDPUTCMD	; 
		CLRF	AX_LCD_counter	; vynulovat temp_reg
		RETURN			; hotovo

AX_LCD_Print:				;
		INCF	AX_LCD_counter, F	; zvysit pocitadlo
		
		MOVF	LCD_TYP, W	; 2x16 nebo 4x20??
		SUBLW	a'4'		; a'4' = 4x20
		BTFSS	STATUS, Z	; preskoc, pokud 4x20
		GOTO	LCD2x16		;

		MOVF	AX_LCD_counter, W	; vzit pocitadlo,
		SUBLW	d'80'		; jsme jiz za pozici 80? (80-W)
		BTFSS	STATUS, C	; preskoc, pokud ne (C=1)
		RETURN			; W>80 => C=0, dal netiskneme
		
		MOVF	AX_LCD_counter, W	; vzit pocitadlo
		XORLW	d'21'		; jsme prave na pozici 21?
		BTFSS	STATUS, Z	; preskoc, pokud ano
		GOTO	LCD4x20_3	; test na radek 3
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
 		CALL	LCDPUTCMD	; protoze jsme prave na 21. pozici
		GOTO	AX_LCD_Printing	; poslat znak na LCD
LCD4x20_3:				;
		MOVF	AX_LCD_counter, W	; vzit pocitadlo
		XORLW	d'41'		; jsme prave na pozici 41?
		BTFSS	STATUS, Z	; preskoc, pokud ano
		GOTO	LCD4x20_4	; test na radek 4
		MOVLW	b'10010100'	; nastavit kursor na 1. byte 3. radku
 		CALL	LCDPUTCMD	; protoze jsme prave na 41. pozici
		GOTO	AX_LCD_Printing	; poslat znak na LCD
LCD4x20_4:				;
		MOVF	AX_LCD_counter, W	; vzit pocitadlo
		XORLW	d'61'		; jsme prave na pozici 61?
		BTFSS	STATUS, Z	; preskoc, pokud ano
		GOTO	AX_LCD_Printing	; poslat znak na LCD
		MOVLW	b'11010100'	; nastavit kursor na 1. byte 4. radku
 		CALL	LCDPUTCMD	; protoze jsme prave na 61. pozici
		GOTO	AX_LCD_Printing	; poslat znak na LCD

LCD2x16:				; LCD je 2x16
		MOVF	AX_LCD_counter, W	; vzit pocitadlo,
		SUBLW	d'32'		; jsme jiz za pozici 32? (32-W)
		BTFSS	STATUS, C	; preskoc, pokud ne (C=1)
		RETURN			; W>32 => C=0, dal netiskneme
		
		MOVF	AX_LCD_counter, W	; vzit pocitadlo
		XORLW	d'17'		; jsme prave na pozici 17?
		BTFSS	STATUS, Z	; preskoc, pokud ano
		GOTO	AX_LCD_Printing	; poslat znak na LCD
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
 		CALL	LCDPUTCMD	; protoze jsme prave na 17. pozici
		
AX_LCD_Printing:			;
		MOVF	send_reg, W	; vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		RETURN			; hotovo

		; Hlavni Prg.

temp		EQU	034H		; Registr docasny pro vse mozne
temp2		EQU	035H		; Registr docasny pro vse mozne
FLAGS_1		EQU	03EH		; Registr priznaku
AX25_SEL	EQU	005H		; Port RA5 (GPS=1 nebo AX25dec=1->0->1)
LCDscr		EQU	000H		; FLAGS_1,0 pro rozliseni LCD obrazovky
SERIAL_SPEED	EQU	001H		; FLAGS_1,1 rychlost RS-232
Only_SMT	EQU	002H		; FLAGS_1,2 jen mereni teploty
Bit_Last_State	EQU	003H		; FLAGS_1,3 posledni stav bitu z MX614
Bit_Now_State	EQU	004H		; FLAGS_1,4 aktualni stav bitu z MX614
WX_func		EQU	007H		; FLAGS_1,7 indikace WX-LCD funkce

LED_flash:				; bliknout LEDkama (pri startu)
		BSF	PORTB,LED	; rozsvitit LED
		CALL	LED_flash_Delay	; pockat chvili
		BCF	PORTB,LED	; zhasnout LED
		CALL	LED_flash_Delay	; pockat chvili
		RETURN			; Return from subroutine


LED_flash_Delay:			; cekaci smycka pro bliknuti LED
		NOP			; 
		DECFSZ	temp,F		; smycka 256x
		GOTO	LED_flash_Delay	;
		DECFSZ	temp2,F		; smycka 256x256
		GOTO	LED_flash_Delay	;
		RETURN			; Return from subroutine


StartGPS:					; Start programu
		CLRF	INTCON		; Zakaz vsech preruseni
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		MOVLW	07H		; odpojit 
		MOVWF	CMCON		; komparatory od A0-A3
		CALL	serial_Init	; Inicializace seriovych portu
		CALL	NMEA_Init	; Inicializace NMEA subsystemu
		;CALL	LCD_Init	; Inicializace LCD subsystemu
		CALL	WX_Start_Init	; Inicializace WX registru
		CALL	WX_RAM_Init	; Inicializace default WX dat
		CALL	Mereni_Init	; Priprava pro nove mereni
		CALL	Start_Mereni	; A zmerit - zjistit pripojeni cidla
		CALL	Start_Mereni	; Nutno volat 4x
		CALL	Start_Mereni	; aby byl vysledek
		CALL	Start_Mereni	; spravne ve stupnich
		CALL	LED_flash	; inicializace probehla
		CALL	Send_Version_LCD	; poslat verzi na LCD
		CALL	SMT_Yes_No_LCD	; Zobrazit (ne) pritomnost LCD

Startup_test:				; Test na stisknuti tlacitek
		BTFSC	PORTA, AX25_SEL	; Stisknut AX25_SEL?
		GOTO	AX25_SEL_not_pressed	; Ne!
		BTFSS	PORTA, SCR_SEL	; AX25_SEL stisknut, co SCR_SEL?
		GOTO	Both_pressed	; Oba stisknuty!
		BCF	FLAGS_1, Only_SMT	; Reset priznaku
		BCF	FLAGS_1, SERIAL_SPEED	; Reset priznaku
		GOTO	Startup_test_end	; A hotovo

Both_pressed:				; Stisknuty oba!
		BSF	FLAGS_1, SERIAL_SPEED	; Stisknut jen SCR_SEL!
		GOTO	Startup_test_end	; A hotovo

AX25_SEL_not_pressed:			; AX25_SEL nestisknut!
		BTFSC	PORTA, SCR_SEL	; Stisknut SCR_SEL?
		GOTO	None_pressed	; Nestisknuto nic!
		BSF	FLAGS_1, Only_SMT	; Stisknut jen AX25_SEL!
		GOTO	Startup_test_end	; A hotovo

None_pressed:				; Nestisknuto nic!
		GOTO	Startup_test_end	; A hotovo

Startup_test_end:			;
		CALL	LED_flash	; aby bylo videt, ze
		CALL	Speed_LCD	; Zobrazit rychlost RS-232 na LCD
		GOTO	Send_Version	; poslat verzi po RS232ce


Main_Loop:				; Hlavni smycka prg, cte z RS232
		BTFSC	FLAGS_1, WX_func	; Jsme v zobrazovani WX?
		GOTO	WX_Buttons_test	; jsme
		BTFSS	PORTA, AX25_SEL	; Nemame prepnout na AX25 dekoder?
		GOTO	StartAX25dec	; Ano, prepnout na AX25 dekoder!
		BTFSC	FLAGS_1, Only_SMT	; Budeme jen merit?
		GOTO	SMT_M		; Ano, jen merit
Main_Loop_Receive:			; 
		CALL	serial_Receive	; cist z RS232
		BTFSS	serialData,00	; prislo neco?
		GOTO	Main_Loop	; neprislo, cekame dal
		MOVWF	temp		; prislo, ulozit to sem
		XORLW	01BH		; je to ESC ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Serial_Receive_ESC	; prisel ESC, co prijde dal?
		MOVF	temp,W		; ESC to nebylo
		CALL	NMEA_Decode	; snad to bude NMEA
		GOTO	Main_Loop	; a jedeme dokolecka

WX_Buttons_test:			; 
		BTFSC	PORTA, RES_MAX	; Stisknut RES_MAX?
		GOTO	RES_MAX_not_pressed	; Ne!
		BTFSS	PORTA, RES_MIN	; RES_MAX stisknut, co RES_MIN?
		GOTO	Both_WX_pressed	; Oba stisknuty!
		CALL	Prepis_TempCMAX_SUBR_LCD	; Prepsat maximum
		GOTO	WX_Buttons_test_end	; A hotovo

Both_WX_pressed:				; Stisknuty oba!
		BCF	FLAGS_1, WX_func	; Reset priznaku
		CALL	LED_flash	; aby bylo videt, ze
		CALL	LED_flash	; funkce WX skoncila
		CALL	Send_Version_LCD	; poslat verzi na LCD
		;CALL	SMT_Yes_No_LCD	; Zobrazit (ne) pritomnost LCD
		CALL	Speed_LCD	; Zobrazit rychlost RS-232 na LCD
		GOTO	Send_Version	; poslat verzi po RS232ce

RES_MAX_not_pressed:			; RES_MAX nestisknut!
		BTFSC	PORTA, RES_MIN	; Stisknut RES_MIN?
		GOTO	WX_Buttons_test_end	; Nestisknuto nic!
		CALL	Prepis_TempCMIN_SUBR_LCD	; Prepsat minimum
		GOTO	WX_Buttons_test_end	; A hotovo

WX_Buttons_test_end:			; A hotovo
		GOTO	Main_Loop_Receive	; a jedeme dokolecka

Serial_Receive_ESC:			; prislo ESC, zpracovavame asi prikaz
		CALL	serial_Receive_BYTE	; cist z RS232
		MOVWF	temp		; ulozit to sem
		XORLW	01BH		; prislo ESC ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Serial_Receive_ESC	; prislo ESC, cekame na prikaz
		MOVF	temp,W		; ESC to nebylo, zjistit prikaz
		XORLW	044H		; prislo D ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Write_Config_to_EEPROM	; 'D'=zapis konfigurace
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	055H		; prislo U ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Config_from_EEPROM	; 'U'=precteni konfigurace
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	056H		; prislo V ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Version	; 'V'=poslat verzi FirmWare
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	052H		; prislo R ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	RAMsend		; 'R'=precteni obsahu RAM registru
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	057H		; prislo W ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	WX_RAMsend	; 'W'=precteni obsahu WX RAM registru 
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	077H		; prislo w ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Write_WX_to_RAM	; 'w'=zapsani do WX RAM registru
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	a'M'		; prislo M ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	SMT_M		; 'M'= mereni teploty na LCD
		MOVF	temp,W		; obnovit prijaty znak
		XORLW	072H		; prislo r ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	GPS_RAMsend	; 'r'=precteni obsahu GPS RAM registru 
		GOTO	Main_Loop	; bylo to neco jineho, ignorovat to

SMT_M:					;
		BCF	FLAGS_1, WX_func	; Shodit priznak WX-LCD funkce
		CALL	Mereni_Init	; Priprava pro nove mereni
		CALL	Start_Mereni	; A zmerit - zjistit pripojeni cidla
		CALL	Start_Mereni	; Nutno volat 4x
		CALL	Start_Mereni	; aby byl vysledek
		BSF	PORTB, LED	; rozsvitit
		CALL	Start_Mereni	; spravne ve stupnich
		BCF	PORTB, LED	; zhasnout
		CALL	SMT_Yes_No_LCD	; Zobrazit (ne) pritomnost LCD
		BTFSS	PORTA, SCR_SEL	; Stisknut SCR_SEL?
		BCF	FLAGS_1, Only_SMT	; ANO! Reset priznaku
		GOTO	Main_Loop	; a zpet do hlavni smycky


Write_Config_to_EEPROM:			; zapise data z RS232 do EEPROM
		CLRF	temp		; vynulovat ukazovatko

Write_Config_to_EEPROM_Loop:		; smycka zapisu do EEPROM
		CALL	serial_Receive_BYTE	; prijmout byte z RS232
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVWF	EEDATA		; ulozit ho sem
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		MOVF	temp,W		; sem dat adresu bytu v EEPROM
		CALL	EEPROM_Write_Byte	; zapsat byte do EEPROM
		INCF	temp,F		; posun na dalsi byte
		BTFSS	temp,EEPROM_size_bit	; bylo jich uz 128?
		GOTO	Write_Config_to_EEPROM_Loop	; jeste ne, dalsi
		GOTO	Main_Loop	; casovani a jedeme dokolecka

Send_Config_from_EEPROM:		; nacte data z EEPROM a posle na RS232
		MOVLW	EE_LEN		; delka dat v EEPROM
		CALL	serial_SEND_byte	; a poslat ji jako prvni
		CLRF	temp		; vynulovat ukazovatko

Send_Config_from_EEPROM_Loop:		; smycka cteni z EEPROM
		MOVF	temp,W		; vzit adresu bytu v EEPROM
		CALL	EEPROM_Read_Byte	; nacist ho
		CALL	serial_SEND_byte	; a poslat po RS232
		INCF	temp,F		; posun na dalsi byte
		BTFSS	temp,EEPROM_size_bit	; bylo jich uz 128?
		GOTO	Send_Config_from_EEPROM_Loop	; jeste ne, dalsi
		GOTO	Main_Loop	; vse odeslano, jedeme dokolecka

Send_Version:				; odesle po startu text verze 
		CLRF	temp		; po RS232, vynulovat temp_reg

Send_Version_serial:			; po RS232 odesle text verze FW
		MOVF	temp,W		; offset bytu verze
		CALL	Version_Data	; ziskat byte verze
		IORLW	000H		; je to binarni nula?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Version_just_send_serial	; je, cely text poslan
		CALL	serial_SEND_byte	; poslat byte
		INCF	temp,F		; posun na dalsi byte
		GOTO	Send_Version_serial	; a zpracovat ho

Version_just_send_serial:		;
		CALL	serialTXcr	; 
		CALL	serialTXlf	; 
		GOTO	Main_Loop	; a zpet do hlavni smycky

		; Odesle obsah RAMky (dopsano)

RAMsend:				; posle obsah registru po RS232
		clrf	temp		; vynulovat ukazovatko

dalsiRAMB:				; odesilaci smycka
		movf	temp,W		; vzit adresu registru
		MOVWF	FSR		; dat ji sem a
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		CALL	serial_SEND_byte	; odeslat po RS-232
		INCF	temp,F		; dalsi REG
		BTFSS	STATUS,Z	; poslany vsechy REGistry?
		GOTO	dalsiRAMB	; jeste ne
		GOTO	Main_Loop	; poslano vse, zpet do hlavni smycky

GPS_RAMsend:				; posle obsah registru po RS232
		CALL	GPS_RAM_send	;
		GOTO	Main_Loop	; poslano vse, zpet do hlavni smycky


GPS_RAM_send:				; posle obsah registru po RS232
		MOVLW	NMEA_STRING_BASE ; od NMEA_STRING_BASE
		MOVWF	FSR		; cist registry

dalsiGPSRAMB:				; odesilaci smycka
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		CALL	serial_SEND_byte	; odeslat po RS-232
		INCF	FSR, F		; dalsi REG
		MOVF	FSR,W		; az do
		XORLW	NMEA_STRING_BASE + NMEA_STRING_LEN + 1 ; konec GPS reg.
		BTFSS	STATUS, Z	; poslany vsechy REGistry?
		GOTO	dalsiGPSRAMB	; jeste ne
		CALL	serialTXcr	; 
		CALL	serialTXlf	; 
		RETURN			; konec


;
;
;       AX.25 rx routine                        N0QBH   2002
;	Rewritten for PIC16F627
;	Digi flag (*) 
;	3.5795 Mhz XTAL shared with MX614
;	Uses TMR1 and all on board memory
;	LED attached to serial send function
;	12/30/02		
;	Ver	 2.6			 Tested Functional
;------------------------------------------------------------
;        LIST    P=16F627
;	ERRORLEVEL	-302
;        include <P16F627.INC>
;        __CONFIG        h'3d21'
;-------------------------------------------------------------
        cblock  0x20
                var_1                   ;various counts (20)
                var_2                   ;various counts
                byte_count              ;offset counter
                store_count             ;bytes stored cnt
                bit_stuff_count         ;packet bitstuff counter
                bit_count               ;packet bit counter
                loop_count              ;prgm loop counter
		offset			;memory store offset
                send_reg                ;serial data
                FLAGS                   ;various flags
                byte_in                 ;data byte rx'd
                AX25_crc_hi             ;upper byte CRC word
                AX25_crc_lo             ;lower byte CRC word
                next_to_last_byte       ;2nd from last byte
                last_byte               ;last byte rxed
                ssid_buffer             ;SSID storage (2f)
		AX_LCD_counter		;Pocitadlo pro zobrazeni na LCD
	endc
;------------------------------------------------------------
;
#define	RAMH_FLAG		FLAGS,0		;bank 0,1/2,3
#define DATA_FLAG               FLAGS,1         ;beg/end flag
#define MX614_IN                PORTA,1         ;MX614 RXD output
#define LCD_PRINT		FLAGS_1,5	; FLAGS_1,5 AX25 znak na LCD?
#define CRC_OFF			FLAGS_1,6	; FLAGS_1,6 vypnuti AX25 CRC
;
ONE_BIT			equ	d'745'		;1 bit w/TMR1/1   (3.579545MHz)
ONE_5_BIT		equ	d'1117'		;1 1/2 bits       (3.579545MHz)
PACKET_FLAG		equ	0x7e		;01111110
MINIMUM_LENGTH		equ	d'15'		;smallest packet in bytes
;
TRIG_A			equ	d'62'		;end of bank 0 memory
TRIG_B			equ	d'142'		;end of bank 1 memory
TRIG_C			equ	d'190'		;end of bank 2 memory
;
FIXED_DELAY_LENGTH	equ	d'56'		;250 uS
;------------------------------------------------------------

StartAX25dec:
        bsf     STATUS,RP0              ;switch to bank 1
	movlw	b'11000001'
        movwf   OPTION_REG              ;TMR0/4
        bcf     STATUS,RP0              ;bank 0
;
	movlw	b'00000101'
	movwf	T1CON			;TMR1/1, on
;
	movlw	b'00000000'
	movwf	INTCON			; reset INTCON
;
	BCF	CRC_OFF			; Impicitne zapnout CRC test
        movlw   d'80'                  ;80 loops
        call    adjustable_delay        ;20 mS delay
	CALL	LED_flash		; inicializace probehla
        call    startup_message         ;display start msg
	CALL	Send_AXvers_LCD		; display start msg on LCD
	CALL	LED_flash		; inicializace probehla
;
;
;------------------------------------------------------------
; Named registers are cleared and the program looks for the 1st
; packet flag to syncronize on.
;
house_clean                             ;new packet
	bcf	PORTB, LED		;LED off
	movlw	0x20
	movwf	FSR			;reset FSR pointer
house_clean_loop
	clrf	INDF			;clear named registers
	incf	FSR,W
	sublw	0x30
	bnc	$+3
	incf	FSR,F
	goto	house_clean_loop
;
        movlw   0xff
        movwf   AX25_crc_hi             ;reset CRC calc
        movwf   AX25_crc_lo
	movlw	d'255' - low ONE_5_BIT	;1.5 bit timeout
	movwf	TMR1L 			;load TMR1 low reg
	movlw	d'255' - high ONE_5_BIT
	movwf	TMR1H			;load TMR1 hi reg				
	bcf	PIR1,TMR1IF		;reset TMR1 ovrfl flag

find_start_flag				;look for 7e b'01111110'
	BTFSS	PORTA, SCR_SEL		; Prechod do GPS-track modu?
	GOTO	StartGPS		; Ano, prechod do GPS-tracku!
	BTFSS	PORTA, AX25_SEL		; Stisknut AX25_SEL?
	BSF	CRC_OFF			; Ano, vypnout CRC test
        call    recv_one_bit            ;check for toggle
        rrf     byte_in,F               ;roll carry in (0 or 1)
        movfw   byte_in                 ;byte_in to W
        xorlw   PACKET_FLAG             ;is it a flag ?
        skpz
        goto    find_start_flag         ;no
	bsf	PORTB, LED		;LED on
;
;------------------------------------------------------------
; This routine creates a received byte from 8 received bits.
; no tone change (1) setc	tone change  (0) clrc
; The carry bit is rotated into byte_in 8 times.
;
get_next_byte				;saw start flag
        clrf    byte_in                 ;clear input buffer
        movlw   0x08
        movwf   bit_count               ;get 8 bits
get_next_bit
        call    recv_one_bit
        skpc
        goto    zero_in                 ;0 rx'd
        incf    bit_stuff_count,F       ;inc counter, 1 rx'd
roll_em_in
        rrf     byte_in,F               ;store carry bit
        movfw   bit_stuff_count
        xorlw   0x05                    ;5 1's in a row ?
        skpnz
        goto    check_stuff             ;yes
count_em_out                            ;no
        decfsz  bit_count,F             ;8 bits ?
        goto    get_next_bit            ;no
        goto    CRC_save_store          ;yes
zero_in
        clrf    bit_stuff_count         ;0 seen, clear bit_stuff_count
        goto    roll_em_in		;store the zero
;------------------------------------------------------------
; Bit stuff decode routine
; This routine handles the bit stuffing which occurs when more
; than 5 bits elapse without a tone change (1's). The sender
; inserts a 0 that is discarded on the receive end. 
; Flags are also detected here (01111110).					
;
check_stuff
        call    recv_one_bit            ;wait 1 bit
        skpnc                           ;if data=0, skip next
        goto    sixth_one               ;unchanged, flag ?
        clrf    bit_stuff_count         ;0 seen, clear count
        goto    count_em_out            ;don't count this bit
sixth_one
        call    recv_one_bit            ;get 6th bit
        skpnc                           ;0 = flag ?
        goto    house_clean             ;no, 7th 1, reset
        movfw   bit_count
        xorlw   0x03			;is the 0 in
        skpz                            ;the right place ?
        goto    house_clean             ;no
        clrf    bit_stuff_count         ;yes, flag, clear bstc
        btfss   DATA_FLAG               ;header/tail flag ?
        goto    get_next_byte           ;header flag
        movfw   store_count             ;tail flag
        sublw   MINIMUM_LENGTH          ;how long is packet ?
        skpnc
        goto    house_clean             ; < 16 bytes rx'd
        goto    packet_end              ;it's long enough
;
; Two bytes are buffered before starting CRC calculations.
; When the end flag is seen, the buffered bytes become the
; sent checksum.
;
CRC_save_store
        btfss   DATA_FLAG               ;test data flag
        bsf     DATA_FLAG               ;set data flag
	incf	store_count,F		;buffer byte count
	movfw	store_count
        xorlw   0xff                    ;how long ?
        skpnz
        goto    house_clean             ;255 bytes, reset
        movfw   store_count             ;how long again
        sublw   0x02
        skpc                            ; >2 ?
        call    AX25_CRC_calc           ;calc CRC, store data
        movfw   last_byte               ;last byte to W
        movwf   next_to_last_byte       ;to 2nd to last
        movfw   byte_in                 ;get new data byte
        movwf   last_byte               ;store in last byte
        goto    get_next_byte           ;get next byte
;
;------------------------------------------------------------
;       Invert CRC data and compare to value sent
;
packet_end
	BTFSC	CRC_OFF			; Pokud je 0, pocitat CRC, jinak
	goto terminus			;skip CRC test
        movlw   0xff
        xorwf   AX25_crc_lo,F
        xorwf   AX25_crc_hi,F           ;invert  CRC data
        movfw   next_to_last_byte       ;check lobyte match
        xorwf   AX25_crc_lo,W
        skpz
        goto    house_clean             ;mismatch, reset
        movfw   last_byte
        xorwf   AX25_crc_hi,W           ;check hibyte match
        skpz
        goto    house_clean             ;mismatch, reset
terminus
	movlw	d'2'
	subwf	store_count,F		;realign total count
;------------------------------------------------------------
; Check the digi SSID's for the last "H bit" set which indicates
; the sending digi.
;
find_sending_digi
	movlw	d'255'
	movwf	var_2	  	        ;reset sending digi offset  
	movlw	d'13'
	movwf	offset			;org address SSID loc
;
fsd_loop_up
	call	RAMRead_skip_count
        movwf   ssid_buffer
        btfsc	ssid_buffer,0
	goto	mark_last_digi		;found last digi
	movlw	d'7'
	addwf	offset,F		;next SSID
	goto	fsd_loop_up
mark_last_digi
	movfw	offset
	movwf	var_1			;end of addresses in memory
fsd_loop_down
	movlw	d'13'
	xorwf	offset,W
	bz	read_back_data		;heard direct
	btfsc	ssid_buffer,7		;bit 7 is "H" bit
	goto	sd_found
	movlw	d'7'
	subwf	offset,F
	call	RAMRead_skip_count
        movwf   ssid_buffer
	goto	fsd_loop_down
sd_found
	movfw	offset
	movwf	var_2	 	        ;store loc of sending digi SSID					
;------------------------------------------------------------
read_back_data                          ;send data serially
        movlw   0x07                    ;start of org call
        movwf   offset		        ;in memory
;
	CALL	AX_LCD_Reset		;
	BSF	LCD_PRINT		; org call se na LCD posila
        call    parse_callsign		;send org call & SSID
;
        movlw   '>'                     ;insert r_arrow
        movwf   send_reg
        call    do_serial_send          ;send >
	CALL	AX_LCD_Print		;
;
        clrf    offset		        ;start of destination call
;
	BCF	LCD_PRINT		; dest call se na LCD ne posila
        call    parse_callsign		;send destination call & SSID
;
	movfw	var_1
	xorlw	d'13'			;end of addresses?
	bnz	first_digi		;no
        movlw   0x0d                    ;yes, setup for info field
        movwf   offset                  ;starting address skips packet
        goto    info_field_begin        ;type bytes

first_digi
	BCF	LCD_PRINT		; digis se na LCD ne posila
        movlw   ','                     ;send 1st digi
        movwf   send_reg
        call    do_serial_send          ;send comma
        movlw   d'14'
        movwf   offset                  ;start of 3rd call (1st digi)
;
display_digipeaters
        call    parse_callsign          ;parse / send next
;
	movfw	var_1
	xorwf	offset,W
        bz      info_field_begin        ;end of addresses
        movlw   ','                     ;next address
        movwf   send_reg
        call    do_serial_send          ;send comma 
;
	incf	offset,F		;align w/next callsign
        goto    display_digipeaters
;
info_field_begin
        movlw   ':'
        movwf   send_reg
        call    do_serial_send          ;send :
	;CALL	AX_LCD_Print		;
	movlw	d'2'
	addwf	offset,F		;account for two
	subwf	byte_count,F		;skipped bytes
info_field_loop
	movfw	byte_count		;end of data?
        bz	wrap_it_up		;yes
	incf	offset,F		;no, keep reading
        call    RAMRead
        movwf   send_reg                ;data to send_reg
        call    do_serial_send          ;send byte
	CALL	AX_LCD_Print		;
        goto    info_field_loop         ;end not seen
wrap_it_up
        movlw   0x0d                    ;>, send end
        movwf   send_reg                ;<CR>
        call    do_serial_send
        movlw   0x0a                    ;<LF>
        movwf   send_reg
        call    do_serial_send
        goto    house_clean
;------------------------------------------------------------
int_service
        retfie                          ;return to program
;------------------------------------------------------------
;       Subroutines area
;------------------------------------------------------------
startup_message
        clrf    var_1                   ;offset clear
startup_loop
        movfw   var_1                   ;offset to W
        call    intro_message           ;get offset byte
        movwf   send_reg                ;store
        xorlw   0x00                    ;test for end
        skpnz
        return                          ;done, start prgm
        call    do_serial_send          ;send byte
        incf    var_1,F                 ;inc offset
        goto    startup_loop            ;loop
;------------------------------------------------------------
;       Stores all packet data as received
;
AX25_CRC_calc
;
        call    RAMWrite                ;store in RAM
	incf	offset,F		;inc store offset
;
        movlw   0x08
        movwf   bit_count               ;8 loops
crc_head
        rrf     next_to_last_byte,F     ;get bit for CRC
        call    crc_core                ;do CRC calc
        decfsz  bit_count,F             ;8 counts
        goto    crc_head
        return
;------------------------------------------------------------
crc_core
        movlw   1
        skpnc
        xorwf   AX25_crc_lo, F          ;crc calculation
        clrc
        rrf     AX25_crc_hi, F
        rrf     AX25_crc_lo, F
        skpc
        return
        movlw   H'08'
        xorwf   AX25_crc_lo, F
        movlw   H'84'
        xorwf   AX25_crc_hi, F
        return
;------------------------------------------------------------
;       Assumes offset is correct
;
parse_callsign
        movlw   0x06
        movwf   loop_count              ;6 loops
parse_callsign_loop	
        call    RAMRead                 ;get byte from RAM
        movwf   send_reg                ;byte to send_reg
        clrc
        rrf     send_reg,F              ;right shift to normal
        movfw   send_reg
        xorlw   ' '                     ;space ?
        skpz				;yes, skip next	
        call    do_serial_send          ;no, send byte
	BTFSS	LCD_PRINT		;
	GOTO	no_AX_LCD		;
        movfw   send_reg
        xorlw   ' '                     ;space ?
        skpz				;yes, skip next	
	CALL	AX_LCD_Print		;
no_AX_LCD:				;
	incf	offset,F		;inc offset
        decfsz  loop_count,F            ;loop count - 1
        goto    parse_callsign_loop
;					
	call	RAMRead 		;get stored SSID byte
	movwf	ssid_buffer	                                
        rrf     ssid_buffer,F           ;right shift byte
        movlw   0x0f
        andwf   ssid_buffer,F           ;mask hi nibble
        skpnz
        goto	ssid_is_zero            ;ssid = 0
        movlw   '-'                     ;add dash
        movwf   send_reg
        call    do_serial_send          ;send -
	BTFSC	LCD_PRINT		;
	CALL	AX_LCD_Print		;
        movlw   d'10'                   ;10
        subwf   ssid_buffer,W           ;test magnitude
        skpc
        goto    send_the_ones           ;SSID < 10
        movwf   ssid_buffer             ;ssid buffer => 10
        movlw   "1"                     ;ascii 1 for tens
        movwf   send_reg                ;W to send_reg
        call    do_serial_send          ;send 10
	BTFSC	LCD_PRINT		;
	CALL	AX_LCD_Print		;
send_the_ones
        movfw   ssid_buffer             ;ssid to W
        addlw   0x30                    ;ascii convert
        movwf   send_reg
        call    do_serial_send
	BTFSC	LCD_PRINT		;
	CALL	AX_LCD_Print		;
ssid_is_zero				;do asteric routine
	movfw	var_2
	xorwf	offset,W		;check for loc match to
	bnz	$+6			;sending digi
	movlw	"*"
	movwf	send_reg
	call	do_serial_send		;send * to denote 
	BTFSC	LCD_PRINT		;
	CALL	AX_LCD_Print		;
        return                          ;sending digi
;------------------------------------------------------------
;       Packet bit receive routine
;	Timed with TMR1			no tone change (1)
;	1 bit = 832 uS			tone change  (0)
;       
recv_one_bit                            
	BTFSC	FLAGS_1, Bit_Now_State	; Test posledniho aktualniho stavu bitu
	GOTO	Last_to_High		; 
	GOTO	Last_to_Low		; 

Last_to_Low:				; hotovo
	BCF	FLAGS_1, Bit_Last_State	; Byl 0, nastavit last na 0
	GOTO	Now_State		; 

Last_to_High:				; hotovo
	BSF	FLAGS_1, Bit_Last_State	; Byl 1, nastavit last na 1
	GOTO	Now_State		; 

Now_State:				;
	BTFSC	MX614_IN		; Test nyni aktualniho stavu bitu
	GOTO	Now_is_High		;
	GOTO	Now_is_Low		;

Now_is_High:				;
	BSF	FLAGS_1, Bit_Now_State	; Je 1, nastavit now na 1
	BTFSS	FLAGS_1, Bit_Last_State	; Byl posledni stav take 1?
	GOTO	bit_edge		; Ne, Bit se zmenil
	GOTO	no_change		; Ano, Bit se nezmenil

Now_is_Low:				;
	BCF	FLAGS_1, Bit_Now_State	; Je 0, nastavit now na 0
	BTFSC	FLAGS_1, Bit_Last_State	; Byl posledni stav take 0?
	GOTO	bit_edge		; Ne, Bit se zmenil
	GOTO	no_change		; Ano, Bit se nezmenil

no_change
        btfss   PIR1,TMR1IF		;no change, check timeout		
        goto    recv_one_bit            ;not timed out
	movlw	d'255' - low ONE_BIT	;timed out, reset timer
	movwf	TMR1L 	
	movlw	d'255' - high ONE_BIT
	movwf	TMR1H			;reset 1 bit timeout
	bcf	PIR1,TMR1IF		;reset TMR1 ovrfl flag
        setc                            ;no toggle,carry is set
        return
bit_edge
	movlw	d'255' - low ONE_5_BIT
	movwf	TMR1L 	
	movlw	d'255' - high ONE_5_BIT
	movwf	TMR1H			;1.5 bit timeout
	bcf	PIR1,TMR1IF		;reset TMR1 ovrfl flag
        clrc		                ;tone toggle, clear carry
        return
;------------------------------------------------------------
;	Sends serial data w/USART module
do_serial_send
        movfw   send_reg
	GOTO	serial_SEND_byte	;
;------------------------------------------------------------
; Stores up to 205 bytes of data in 3 separate sections of on board RAM.
; Routine counts the bytes and selects which bank of memory to use.
; Both writes and reads use indirect addressing for memory access.
;
RAMWrite
	incf	byte_count,W		;count the bytes in
	sublw	d'190'			;test for => 190
	skpc
	return
	incf	byte_count,F		;< 190, continue
	call	RAM_Calc		;sort which bank to store in
	btfsc	RAMH_FLAG		;store in bank 2?
	goto	hi_write		;yes
	movfw	next_to_last_byte	;no
	movwf	INDF			;store W at indirect offset
	return
hi_write
	movfw	next_to_last_byte
	bsf	STATUS,IRP		;bank 2,3
	movwf	INDF			;store W at indirect offset
	bcf	STATUS,IRP		;bank 0,1
	bcf	RAMH_FLAG		;clear flag
	return
RAMRead
	decf	byte_count,F		;count the bytes out
RAMRead_skip_count
	call	RAM_Calc		;sort which bank to read from
	btfsc	RAMH_FLAG		;read from bank 2?
	goto	hi_read			;yes
	movfw	INDF			;no, read from indirect offset
	return
hi_read
	bsf	STATUS,IRP		;bank 2,3
	movfw	INDF			;read from indirect offset
	bcf	STATUS,IRP		;bank 0,1
	bcf	RAMH_FLAG		;clear flag
	return
;
RAM_Calc				;calcs FSR from offset
;
	movfw	offset
	sublw	TRIG_A			;62
	skpnc
	goto	mem_A			;offset < 62
	movfw	offset
	sublw	TRIG_B			;142
	skpnc
	goto	mem_B			;offset 63 to 142	
	movfw	offset
	sublw	TRIG_C			;190
	skpnc
	goto	mem_C			;offset 143 to 190
	movlw	TRIG_C			;190
	movwf	offset			;hold here
	goto	mem_C
;
mem_A
	movfw	offset			;0 to 62
	addlw	d'65'			;65 to 127
	movwf	FSR		
	return
mem_B
	movfw	offset			;63 to 142
	addlw	d'97'			;160 to 239
	movwf	FSR	
	return
mem_C
	movlw	d'111'			;143 to 190
	subwf	offset,W		;32 to 79
	movwf	FSR
	bsf	RAMH_FLAG		;use bank 2
	return
;
;------------------------------------------------------------
;		 3.579 Mhz version
fixed_delay                             ;250 uS
        movlw   FIXED_DELAY_LENGTH      ;4.47 uS per loop
        movwf   var_2
	nop
        decfsz  var_2,F
        goto    $-2
        return
;------------------------------------------------------------
adjustable_delay                        ;W contains length
        movwf   var_1
        call    fixed_delay		;wait 250 uS per count
        decfsz  var_1,F
        goto    $-2
        return
;------------------------------------------------------------
;        END
;------------------------------------------------------------
;Copyright (c) 2004, Mike Berg N0QBH
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, 
;are permitted provided that the following conditions are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright notice, 
;this list of conditions and the following disclaimer in the documentation 
;and/or other materials provided with the distribution. 
;Neither the name of the <ORGANIZATION> nor the names of its contributors 
;may be used to endorse or promote products derived from this software without 
;specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
;AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
;THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
;PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
;BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
;SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
;INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
;ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;THE POSSIBILITY OF SUCH DAMAGE.
;------------------------------------------------------------
;
;
;
		; WX-LCD.ASM

RES_MAX		EQU	005H		; SW na RA5 je reset MAXima teploty
RES_MIN		EQU	000H		; SW na RA0 je reset MINima teploty

LCD_WX_ID_TF	EQU	07AH		; registr
LCD_WX_ID_TC	EQU	07BH		; registr
LCD_WX_ID_P	EQU	07CH		; registr
LCD_WX_ID_H	EQU	07DH		; registr

LCD_WX_STRT	EQU	07EH		; registr
wx_data_endR	EQU	07FH		; Registr s ukonc. znakem WX retezce

WX_Start_Init:
		BSF	STATUS,RP0		; Select page 1 (80H - FFH)
		MOVLW	EE_WX_DATA_END		; adresa ukonc. znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	wx_data_endR		; a ulozit ji sem
		MOVLW	EE_WX_DATA_STRT		; adresa start znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_WX_STRT		; a ulozit ji sem
		MOVLW	EE_WX_ID_TF		; adresa Farhent. znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_WX_ID_TF		; a ulozit ji sem
		MOVLW	EE_WX_ID_TC		; adresa Celsi. znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_WX_ID_TC		; a ulozit ji sem
		MOVLW	EE_WX_ID_P		; adresa tlak. znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_WX_ID_P		; a ulozit ji sem
		MOVLW	EE_WX_ID_H		; adresa vlhk. znaku WX retezce
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	LCD_WX_ID_H		; a ulozit ji sem
		MOVLW	a'9'			; ascii '9' (39h)
		MOVWF	LCD_TempCMAXd		; ulozit sem
		MOVWF	LCD_TempCMAXj		; ulozit sem
		MOVWF	LCD_TempCMAXdes		; ulozit sem
		MOVWF	LCD_TempCMINd		; ulozit sem
		MOVWF	LCD_TempCMINj		; ulozit sem
		MOVWF	LCD_TempCMINdes		; ulozit sem
		MOVLW	a'-'			; ascii '-' (2Dh)
		MOVWF	LCD_TempCMAXz		; ulozit sem
		MOVLW	a' '			; ascii ' ' (20h)
		MOVWF	LCD_TempCMINz		; ulozit sem
		RETURN				; Return from subroutine

		; WX Po RS232 obsluha

WX_temp		EQU	020H		; RP0=1, temp
WX_DATA_OFFSET	EQU	021H		; RP0=1, offset WX ramce v RAM
WX_DATA_END	EQU	00DH		; koncovy znak WX dat

Write_WX_to_RAM:			; zapsat WX do RAMky
		BSF	FLAGS_1, WX_func	; Jsme v zobrazovani WX!
		BSF	PORTB, LED	; Rozsvitit WX LEDku
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVLW	WX_DATA_OFFSET	; zacatek WX retezce
		MOVWF	WX_temp		; dat sem

Write_WX_to_RAM_Loop:			; smycka zapisu WX do RAM banky 1
		MOVF	WX_temp,W	; vzit adresu registru
		MOVWF	FSR		; dat ji sem a
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		CALL	serial_Receive_BYTE	; prijmout byte z RS232
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		BSF	FSR,7
		MOVWF	INDF		; zapsat byte na adresu FSR
		BCF	FSR,7
		INCF	WX_temp,F	; dalsi REG

		BTFSS	WX_temp,6	; nepretekli
		GOTO	WX_end_W_check	; jsme 
		BTFSS	WX_temp,5	; nepretekli
		GOTO	WX_end_W_check	; jsme 
		BTFSC	WX_temp,4	; v RAMce?
		GOTO	WX_end_W	; pretekli, konec 

WX_end_W_check:
		XORWF	wx_data_endR, W	; ukoncovaci znak WX retezce
		BTFSS	STATUS,Z	; prijaty vsechy WX registry?
		GOTO	Write_WX_to_RAM_Loop	; jeste ne, dalsi

WX_end_W:	
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)

		; Mame WX data, jdeme je zobrazit na LCD

		CALL	Pripr_WX_LCD	; Pripravit data pro LCD
		CALL	Zobr_WX_na_LCD	; A poslat data na LCD
		BCF	PORTB, LED	; Zhasnout WX LEDku
		GOTO	Main_Loop	; hotovo a jedeme dokolecka

Pripr_WX_LCD				; Pripravi data pro LCD

LCD_temp0	EQU	020H		; registr pomocny

LCD_TempCz	EQU	021H		; registr
LCD_TempCd	EQU	022H		; registr
LCD_TempCj	EQU	023H		; registr
LCD_TempCdes	EQU	024H		; registr
LCD_TempCMAXz	EQU	025H		; registr
LCD_TempCMAXd	EQU	026H		; registr
LCD_TempCMAXj	EQU	027H		; registr
LCD_TempCMAXdes	EQU	028H		; registr
LCD_TempCMINz	EQU	029H		; registr
LCD_TempCMINd	EQU	02AH		; registr
LCD_TempCMINj	EQU	02BH		; registr
LCD_TempCMINdes	EQU	02CH		; registr

LCD_HumiD	EQU	02DH		; registr
LCD_HumiJ	EQU	02EH		; registr

LCD_PresT	EQU	02FH		; registr
LCD_PresS	EQU	030H		; registr
LCD_PresD	EQU	031H		; registr
LCD_PresJ	EQU	032H		; registr
LCD_PresDes	EQU	033H		; registr
		; 000/000g000t000b00000h00<CR>-000C
		; LCD_WX_STRT
		; LCD_WX_ID_TF
		; LCD_WX_ID_TC
		; LCD_WX_ID_P
		; LCD_WX_ID_H
		; wx_data_endR

		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVLW	WX_DATA_OFFSET	; zacatek WX retezce
		MOVWF	WX_temp		; dat sem

WX_dalsiRAMB_LCD:				; odesilaci smycka
		MOVF	WX_temp,W	; vzit adresu registru
		MOVWF	FSR		; dat ji sem a
		BSF	FSR,7
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		BCF	FSR,7
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		MOVWF	LCD_temp0	; Ulozit znak sem

		MOVF	LCD_temp0,W	; obnovit prijaty znak
		XORWF	LCD_WX_ID_TC, W	; prislo ID Teploty C?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	LCD_TempC_prep	; pripravit teplotu
		
		MOVF	LCD_temp0,W	; obnovit prijaty znak
		XORWF	LCD_WX_ID_P, W	; prislo ID Tlaku?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	LCD_Pres_prep	; pripravit teplotu
		
		MOVF	LCD_temp0,W	; obnovit prijaty znak
		XORWF	LCD_WX_ID_H, W	; prislo ID Vlhkosti?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	LCD_Humi_prep	; pripravit teplotu
		
WX_dalsiRAMB_LCD_RET:			; sem skok z "podprogramu"
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		INCF	WX_temp,F	; dalsi REG

		BTFSS	WX_temp,6	; nepretekli
		GOTO	WX_end_LCD_check	; jsme 
		BTFSS	WX_temp,5	; nepretekli
		GOTO	WX_end_LCD_check	; jsme 
		BTFSC	WX_temp,4	; v RAMce?
		GOTO	WX_end_LCD	; pretekli, konec 

WX_end_LCD_check:
		BSF	FSR,7
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		BCF	FSR,7
		XORWF	wx_data_endR, W	; ukoncovaci znak WX retezce
		BTFSS	STATUS,Z	; prohledany vsechy WX registry?
		GOTO	WX_dalsiRAMB_LCD	; jeste ne
WX_end_LCD:
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)

		RETURN			; navrat

LCD_TempC_prep					;
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_TempCz		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_TempCd		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_TempCj		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_TempCdes		; Ulozit znak sem

Test_TempCMAX					; test mame-li maximum
		MOVF	LCD_TempCz, W		; vzit znamenko (20H/2DH)
		SUBWF	LCD_TempCMAXz, W	; odecist aktualni od MAX
		BTFSS	STATUS, C		;
		GOTO	Test_Temp_CMIN		; MAX je +, AKT je -
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je +, MAX je -

		BTFSC	LCD_TempCz, 0		; je znamenko kladne? (20h)
		GOTO	Test_TempCMAX_zap	; ne, je zaporne (2Dh)

		MOVF	LCD_TempCMAXd, W	; vzit MAX desitky 
		SUBWF	LCD_TempCd, W		; odecist MAX od aktualni
		BTFSS	STATUS, C		; kdyz AKT < MAX ... C=0
		GOTO	Test_Temp_CMIN		; AKT je mensi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je vetsi nez MAX
		MOVF	LCD_TempCMAXj, W	; vzit MAX jednotky 
		SUBWF	LCD_TempCj, W		; odecist MAX od aktualni
		BTFSS	STATUS, C		; kdyz AKT < MAX ... C=0
		GOTO	Test_Temp_CMIN		; AKT je mensi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je vetsi nez MAX
		MOVF	LCD_TempCMAXdes, W	; vzit MAX desetiny 
		SUBWF	LCD_TempCdes, W		; odecist MAX od aktualni
		BTFSS	STATUS, C		; kdyz AKT < MAX ... C=0
		GOTO	Test_Temp_CMIN		; AKT je mensi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je vetsi nez MAX
		GOTO	Test_Temp_CMIN		; AKT je cele shodne s MAX
		
Test_TempCMAX_zap				; teplota pod nulou
		MOVF	LCD_TempCd, W		; vzit AKT desitky
		SUBWF	LCD_TempCMAXd, W	; odecist aktualni od MAX
		BTFSS	STATUS, C		; 
		GOTO	Test_Temp_CMIN		; AKT je zapornejsi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je kladnejsi nez MAX
		MOVF	LCD_TempCj, W		; vzit AKT jednotky
		SUBWF	LCD_TempCMAXj, W	; odecist aktualni od MAX
		BTFSS	STATUS, C		;
		GOTO	Test_Temp_CMIN		; AKT je zapornejsi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je kladnejsi nez MAX
		MOVF	LCD_TempCdes, W		; vzit AKT desetiny
		SUBWF	LCD_TempCMAXdes, W	; odecist aktualni od MAX
		BTFSS	STATUS, C		;
		GOTO	Test_Temp_CMIN		; AKT je zapornejsi nez MAX
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMAX		; AKT je kladnejsi nez MAX
		GOTO	Test_Temp_CMIN		; AKT je cele shodne s MAX
		
Prepis_TempCMAX					; mame nove maximum
		CALL	Prepis_TempCMAX_SUBR	; vlastni prepis

Test_Temp_CMIN					; test mame-li minimum
		MOVF	LCD_TempCMINz, W	; vzit MIN znamenko (20H/2DH)
		SUBWF	LCD_TempCz, W		; odecist MIN od aktualniho
		BTFSS	STATUS, C		;
		GOTO	LCD_TempC_prep_END	; MIN je -, AKT je +
		BTFSS	STATUS, Z		; kdyz AKT = MAX ... Z=1
		GOTO	Prepis_TempCMIN		; MIN je +, AKT je -

		BTFSC	LCD_TempCz, 0		; je znamenko kladne? (20h)
		GOTO	Test_TempCMIN_zap	; ne, je zaporne (2Dh)

		MOVF	LCD_TempCd, W		; vzit AKT desitky
		SUBWF	LCD_TempCMINd, W	; odecist aktualni od MIN
		BTFSS	STATUS, C		; kdyz AKT > MIN ... C=0
		GOTO	LCD_TempC_prep_END	; AKT je vetsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je mensi nez MIN
		MOVF	LCD_TempCj, W		; vzit AKT jednotky
		SUBWF	LCD_TempCMINj, W	; odecist aktualni od MIN
		BTFSS	STATUS, C		; kdyz AKT > MIN ... C=0
		GOTO	LCD_TempC_prep_END	; AKT je vetsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je mensi nez MIN
		MOVF	LCD_TempCdes, W		; vzit AKT desetiny
		SUBWF	LCD_TempCMINdes, W	; odecist aktualni od MIN
		BTFSS	STATUS, C		; kdyz AKT > MIN ... C=0
		GOTO	LCD_TempC_prep_END	; AKT je vetsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je mensi nez MIN
		GOTO	LCD_TempC_prep_END	; AKT cele shodne s MIN
		
Test_TempCMIN_zap				; teplota pod nulou
		MOVF	LCD_TempCMINd, W	; vzit MIN desitky 
		SUBWF	LCD_TempCd, W		; odecist MIN od aktualni
		BTFSS	STATUS, C		; 
		GOTO	LCD_TempC_prep_END	; AKT je kladnejsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je zapornejsi nez MIN
		MOVF	LCD_TempCMINj, W	; vzit MIN jednotky 
		SUBWF	LCD_TempCj, W		; odecist MIN od aktualni
		BTFSS	STATUS, C		;
		GOTO	LCD_TempC_prep_END	; AKT je kladnejsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je zapornejsi nez MIN
		MOVF	LCD_TempCMINdes, W	; vzit MIN desetiny 
		SUBWF	LCD_TempCdes, W		; odecist MIN od aktualni
		BTFSS	STATUS, C		;
		GOTO	LCD_TempC_prep_END	; AKT je kladnejsi nez MIN
		BTFSS	STATUS, Z		; kdyz AKT = MIN ... Z=1
		GOTO	Prepis_TempCMIN		; AKT je zapornejsi nez MIN
		GOTO	LCD_TempC_prep_END	; AKT je cele shodne s MIN
		
Prepis_TempCMIN					; mame nove minimum
		CALL	Prepis_TempCMIN_SUBR	; vlastni prepis

LCD_TempC_prep_END				;
		GOTO	WX_dalsiRAMB_LCD_RET	;

LCD_Pres_prep					;
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_PresT		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_PresS		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_PresD		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_PresJ		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_PresDes		; Ulozit znak sem
		GOTO	WX_dalsiRAMB_LCD_RET

LCD_Humi_prep					;
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_HumiD		; Ulozit znak sem
		CALL	LCD_prep_WXzn		; ziskat znak z retezce
		MOVWF	LCD_HumiJ		; Ulozit znak sem
		GOTO	WX_dalsiRAMB_LCD_RET

LCD_prep_WXzn				;
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		INCF	WX_temp,F	; dalsi REG
		MOVF	WX_temp,W	; vzit adresu registru
		MOVWF	FSR		; dat ji sem a
		BSF	FSR,7
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		BCF	FSR,7
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		RETURN			;
		

Prepis_TempCMAX_SUBR			; mame nove maximum
		MOVF	LCD_TempCz, W	; vzit znamenko (20H/2DH)
		MOVWF	LCD_TempCMAXz	; ulozit sem
		MOVF	LCD_TempCd, W	; vzit desitky
		MOVWF	LCD_TempCMAXd	; ulozit sem
		MOVF	LCD_TempCj, W	; vzit jednotky
		MOVWF	LCD_TempCMAXj	; ulozit sem
		MOVF	LCD_TempCdes, W	; vzit desetiny
		MOVWF	LCD_TempCMAXdes	; ulozit sem
		RETURN			;

Prepis_TempCMIN_SUBR			; mame nove minimum
		MOVF	LCD_TempCz, W	; vzit znamenko (20H/2DH)
		MOVWF	LCD_TempCMINz	; ulozit sem
		MOVF	LCD_TempCd, W	; vzit desitky
		MOVWF	LCD_TempCMINd	; ulozit sem
		MOVF	LCD_TempCj, W	; vzit jednotky
		MOVWF	LCD_TempCMINj	; ulozit sem
		MOVF	LCD_TempCdes, W	; vzit desetiny
		MOVWF	LCD_TempCMINdes	; ulozit sem
		RETURN			;

Prepis_TempCMAX_SUBR_LCD		; prepsani maxima
		CALL	Prepis_TempCMAX_SUBR	; 
		GOTO	Zobr_WX_na_LCD	;

Prepis_TempCMIN_SUBR_LCD		; prepsani minima
		CALL	Prepis_TempCMIN_SUBR	; 
		GOTO	Zobr_WX_na_LCD	;

Zobr_WX_na_LCD				; Zobrazi pripraveny WX retezec
		; Teplota 1. radek
		MOVLW	b'10000000'	; nastavit kursor na 1. byte 1. radku
 		CALL	LCDPUTCMD
		MOVF	LCD_TempCz, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_TempCd, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_TempCj, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat  na LCD
		MOVLW	a'.'		; desetina tecka
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_TempCdes, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'C'		; stupne Celsia
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a' '		; mezera
 		CALL	LCDPUTCHAR	; Poslat na LCD
		; Tlak
		MOVF	LCD_PresT, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_PresS, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_PresD, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat  na LCD
		MOVF	LCD_PresJ, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat  na LCD
		MOVLW	a'.'		; desetina tecka
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVF	LCD_PresDes, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'h'		; 'hPa'
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'P'		; 'hPa'
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a'a'		; 'hPa'
 		CALL	LCDPUTCHAR	; Poslat na LCD
		; Vlhkost - 2.radek
		MOVLW	b'11000000'	; nastavit kursor na 1. byte 2. radku
 		CALL	LCDPUTCMD
		MOVF	LCD_HumiD, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat  na LCD
		MOVF	LCD_HumiJ, W	; Vzit znak
 		CALL	LCDPUTCHAR	; Poslat  na LCD
		MOVLW	a'%'		; '%'
 		CALL	LCDPUTCHAR	; Poslat na LCD
		MOVLW	a' '		; mezera
 		CALL	LCDPUTCHAR	; Poslat na LCD
		; MIN a MAX teploty
		MOVF	LCD_TempCMINz, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMINd, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMINj, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat  na LCD
		MOVLW	a'.'			; desetina tecka
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMINdes, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVLW	a'C'			; stupne Celsia
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMAXz, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMAXd, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMAXj, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat  na LCD
		MOVLW	a'.'			; desetina tecka
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVF	LCD_TempCMAXdes, W	; Vzit znak
 		CALL	LCDPUTCHAR		; Poslat na LCD
		MOVLW	a'C'			; stupne Celsia
 		CALL	LCDPUTCHAR		; Poslat na LCD
		RETURN			; navrat
		
WX_RAMsend:				; posle obsah WX registru po RS232
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVLW	WX_DATA_OFFSET	; zacatek WX retezce
		MOVWF	WX_temp		; dat sem

WX_dalsiRAMB:				; odesilaci smycka
		MOVF	WX_temp,W	; vzit adresu registru
		MOVWF	FSR		; dat ji sem a
		BSF	FSR,7
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		BCF	FSR,7
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		CALL	serial_SEND_byte	; odeslat po RS-232
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		INCF	WX_temp,F	; dalsi REG

		BTFSS	WX_temp,6	; nepretekli
		GOTO	WX_end_R_check	; jsme 
		BTFSS	WX_temp,5	; nepretekli
		GOTO	WX_end_R_check	; jsme 
		BTFSC	WX_temp,4	; v RAMce?
		GOTO	WX_end_R	; pretekli, konec 

WX_end_R_check:
		BSF	FSR,7
		MOVF	INDF,W		; nacist obsah REG na adrese FSR
		BCF	FSR,7
		XORWF	wx_data_endR, W	; ukoncovaci znak WX retezce
		BTFSS	STATUS,Z	; poslany vsechy WX registry?
		GOTO	WX_dalsiRAMB	; jeste ne
WX_end_R:
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		call	serialTXcr
		call	serialTXlf
		GOTO	Main_Loop	; poslano vse, zpet do hlavni smycky

WX_RAM_Init:	
		MOVLW	WX_DATA_OFFSET	; zacatek WX retezce v RAM
		MOVWF	FSR		; dat ji sem a
		BSF	FSR,7		; Banka 1 ve FSR
		MOVLW	a'0'		; '000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	002FH		; '/'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'g'		; 'g'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a't'		; 't'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'b'		; 'b'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '00000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'h'		; 'h'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '00'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	000DH		; 'CR'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a' '		; ' '
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'0'		; '000'
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVWF	INDF		; zapsat byte na adresu FSR
		INCF	FSR,F		; dalsi REG
		MOVLW	a'C'		; 'C'
		MOVWF	INDF		; zapsat byte na adresu FSR
		BCF	FSR,7		; Banka 0 ve FSR
		RETURN



		; Pojistky procesoru
		
		ORG	2007H

		DATA	0D02H

		; EEPROM procesoru

EEPROM_size_bit	EQU	007H		; Bitik preteceni EEPROM
EE_LEN		EQU	080H		; velikost EEPROM

		ORG	2100H

EE_WX_DATA_STRT	EQU	001H		; Adresa start. znaku WX retezce
EE_WX_DATA_END	EQU	002H		; Adresa ukonc. znaku WX retezce
EE_WX_ID_TF	EQU	003H		; Adresa ID teploty F
EE_WX_ID_TC	EQU	004H		; Adresa ID teploty C
EE_WX_ID_P	EQU	005H		; Adresa ID tlaku
EE_WX_ID_H	EQU	006H		; Adresa ID vlhkosti 
EE_LCD_TYP	EQU	007H		; Adresa typu LCD

		DATA	000H		; 
		DATA	a'!'		; Startovaci znak WX APRS retezce
		DATA	a'C'		; Ukoncovaci znak WX APRS retezce
		DATA	a't'		; ID teploty F
		DATA	000DH		; ID teploty C
		DATA	a'b'		; ID tlaku
		DATA	a'h'		; ID vlhkosti
		DATA	a'2'		; '2' = 2x16, '4' = 4x20
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 

		END

